/*
 * All content copyright (c) 2003-2009 Terracotta, Inc., except as may otherwise be noted in a separate copyright notice.  All rights reserved.
 */
Ext.ns('TC.monitor.tabs');

TC.monitor.tabs.CacheEfficiency = function(monitor) {
    this.monitor = monitor;

    this.createStores();

    this.createInterface();
    /*
    Ext.override(Ext.grid.GroupingView, {
        getRows : function(){
            if(!this.canGroup()){
                return Ext.grid.GroupingView.superclass.getRows.call(this);
            }
            var r = [],
                gs = this.getGroups(),
                g,
                i = 0,
                len = gs.length,
                j,
                jlen;
            for(; i < len; ++i){
                g = gs[i].childNodes[1];
                if(g){
                    g = g.childNodes;
                    for(j = 0, jlen = g.length; j < jlen; ++j){
                        r[r.length] = g[j];
                    }
                }
            }
            return r;
        }
    });*/

};

TC.monitor.tabs.CacheEfficiency.prototype = {
    monitor: null,

    cacheManagerAddress: null,
    cacheManagerPort: null,
    cacheName: null,

    comboCacheManager: null,
    comboCache: null,
    chartPanel: null,
    chart: null,
    maximum: new Date(),
    minimum: new Date(),
    timeAxisFormat: 'H:i:s',
    forwardButton: null,
    backButton: null,
    settingsButton: null,
    settingsWindow: null,
    xAxisTitle: '',
    comboCacheManagerStore: null,
    comboCacheStore: null,
    chartStore: null,
    live: true,
    chartReady: false,
    fromDate: null,
    fromTime: null,
    timeInterval: 100,
    loadFlag:false,
    runOnce:false,
    statType: 'hitRatio',

    /**
     *  The next few array definitions cover information about the chart type.
     *
     *  statTypeStore: holds the list of possible stores  [ '<name used to call the data/reference> ', '<Plain english for the user to know what they're selecting>' ]
     *  chartTitles: holds the relevant Chart Title to accompany the data    <name used to call the data/reference>: "<plain english title",
     *  yAxisTitles: same as above but for the yAxis label
     *  yAxisFormats: generally leave this one as '0,0' but you are still required to make an entry for every chartType you add.
     *  yAxisMaxes: used to predefine a yAxis max value, in the case of a percent that's 100, if unsure set to 'calculate'
     *      and that will grab the highest value and add a buffer at the top.
     *
     *   removed,
    put,
    updated,
    expired,
    evicted,
    hit,
    hitMemory,
    hitOffHeap,
    hitDisk,
    hitRatio(true),
    miss,
    missMemory,
    missOffHeap,
    missDisk,
    totalCount,
    diskCount,
    offHeapCount,
    memoryCount,
    memorySize,
    offHeapUsage
     *
     */
    statTypeStore: [
        ['hitRatio', 'Hit Ratio (Cache Hits/Total Hits)'],
        ['hit', 'Hits'],
        ['hitMemory','Hits in memory'],
        ['hitOffHeap','Hits off Heap'],
        ['hitDisk', 'Hits on Disk'],
        ['miss', 'Misses'],
        ['missMemory','Misses in memory'],
        ['missOffHeap','Misses off Heap'],
        ['missDisk', 'Misses on Disk'],
        ['put', 'Puts'],
        ['updated', 'Updates'],
        ['removed', 'Removes'],
        ['expired', 'Expiries'],
        ['totalCount', 'Total Elements in Cache'],
        ['diskCount', 'Total Elements on Disk'],
        ['offHeapCount', 'Total Elements OffHeap'],
        ['memoryCount', 'Total Elements in Memory'],
        ['memorySize', 'Estimated memory use for Cache'],
        ['offHeapUsage', 'OffHeap Usage'],
    ],

    chartTitles: {
        hitRatio: 'Hit Ratio (Cache Hits/Total Hits)',
        expired: 'Expiries',
        put: 'Puts',
        updated: 'Updates',
        removed: 'Removes',
        hit: 'Hits',
        hitDisk: 'Hits on Disk',
        hitMemory: 'Hits in memory',
        hitOffHeap: 'Hits off heap',
        miss: 'Misses',
        missDisk: 'Misses on Disk',
        missMemory: 'Misses in memory',
        missOffHeap: 'Misses off heap',
        totalCount: 'Total Elements in cache',
        diskCount: 'Total Elements on disk',
        offHeapCount: 'Total Elements off heap',
        memoryCount: 'Total Elements in memory',
        memorySize: 'Estimated memory use for cache',
        offHeapUsage: 'OffHeap Usage'
    },

    yAxisTitles: {
        hitRatio: '%',
        expired: 'No./s',
        put: 'No./s',
        updated: 'No./s',
        removed: 'No./s',
        hit: 'No./s',
        hitDisk: 'No./s',
        hitMemory: 'No./s',
        hitOffHeap: 'No./s',
        miss: 'No./s',
        missDisk: 'No./s',
        missMemory: 'No./s',
        missOffHeap: 'No./s',
        totalCount: 'No. of Elements',
        diskCount: 'No. of Elements',
        offHeapCount: 'No. of Elements',
        memoryCount: 'No. of Elements',
        memorySize: 'Memory',
        offHeapUsage: '%'
    },

    yAxisFormats: {
        hitRatio: '0,0',
        expired: '0,0',
        put: '0,0',
        updated: '0,0',
        removed: '0,0',
        hit: '0,0',
        hitDisk: '0,0',
        hitMemory: '0,0',
        hitOffHeap: '0,0',
        miss: '0,0',
        missDisk: '0,0',
        missMemory: '0,0',
        missOffHeap: '0,0',
        totalCount: '0,0',
        diskCount: '0,0',
        offHeapCount: '0,0',
        memoryCount: '0,0',
        memorySize: '0,0',
        offHeapUsage: '0,0'
    },
     yAxisMaxes: {
        //Percentage max i.e. 100%
        hitRatio: 100,

        //Calculated these onload for optimal display
        expired: 'calculate',
        put: 'calculate',
        updated: 'calculate',
        removed: 'calculate',
        hit: 'calculate',
        hitDisk: 'calculate',
        hitMemory: 'calculate',
        hitOffHeap: 'calculate',
        miss: 'calculate',
        missDisk: 'calculate',
        missMemory: 'calculate',
        missOffHeap: 'calculate',
        totalCount: 'calculate',
        diskCount: 'calculate',
        offHeapCount: 'calculate',
        memoryCount: 'calculate',
        memorySize: 'calculate',
        offHeapUsage: 100
    },

    xAxis: null,
    yAxis: null,

    loadCacheStore: function() {
        if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null) {
            this.comboCacheStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getCacheNames'});
            this.comboCacheStore.load({
                params: {
                    'address': this.cacheManagerAddress,
                    'port': this.cacheManagerPort
                }
            });
        }
    },

    // TODO Handle the case where the cache manager is set to All in another tab.
    loadChartStore: function(offset) {
        if (this.chartStore !== null && this.chartPanel !== null) {
            if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null && this.cacheName !== null) {
                var params = {
                    'address': this.cacheManagerAddress,
                    'port': this.cacheManagerPort,
                    'cache': this.cacheName,
                    'name': this.statType,
                    'timeInterval': this.timeInterval
                };
                 Ext.getCmp('chartPanel').hide();
                 Ext.getCmp('chartPanel').show();
                this.chartStore.load({
                    params: params,
                    scope: this,
                    callback: function() {
                            var chartPointer = Ext.getCmp('chart');

                            this.chartPanel.setTitle(this.chartTitles[this.statType]);
                            this.loadFlag = true;

                             var Ymax = 100;

                             if(this.yAxisMaxes[this.statType].toString() == 'calculate') {
                                 var max = 0;
                                 var fornum=10;
										
                                 if(this.chartStore.getCount() < 10){
                                     fornum = this.chartStore.getCount();
                                 }
                                 fornum++;
                                 var range =  this.chartStore.getRange(this.chartStore.getCount()-(fornum), this.chartStore.getCount()-1);
                                 range.sort(function(a, b){
                                     return b.data.value - a.data.value;
                                 });
                                 max = range.length > 0 ? range[0].data.value : 0;
                                 Ymax = Math.round(max/10)*10+10;
                              } else{
                                 Ymax  = this.yAxisMaxes[this.statType];
                              }
                              this.yAxis = new Ext.chart.NumericAxis({
                                  title  : this.yAxisTitles[this.statType],  //same as initial title
                                  minimum: this.chart.yAxis.minimum, //make sure it works for multiple changes
                                  maximum: Ymax});
                             if(chartPointer.labelFn) {
                                 chartPointer.setYAxis(this.yAxis);
                             } else {
                                 chartPointer.yAxis = this.yAxis;
                             }
										
                             var xAxisTitle = 'Time - Live - Every ' + this.timeInterval/10 + "sec";
                             this.generateValues();
                             this.xAxisTitle = xAxisTitle;
                             var interval = this.timeInterval;
 
                             this.xAxis = new Ext.chart.TimeAxis({
                                 title: xAxisTitle,
                                 maximum: this.maximum,
                                 minimum: this.minimum,
                                 majorTimeUnit: 'seconds',
                                 majorUnit: this.timeInterval/10,
                                 labelRenderer: function(date) {
                                     //NB - x axis splitting single increment over entire axis
                                     /* if (interval == 600 || interval == 3000) {
                                            return date.format("H:i:s");
                                        } else if (interval == 18000 || interval == 36000) {
                                            return date.format("H:i");
                                        } else if (interval == 864000) {
                                            return date.format("H");
                                        } else if ( interval == 100) { */
                                            return date.format("H:i:s");
                                     /* }  else {
                                            return date.format("H");
                                        }   */
                                 }
                             });
                             if(chartPointer.labelFn) {
                                 chartPointer.setXAxis(this.xAxis);
                             } else {
                                 chartPointer.xAxis = this.xAxis;
                             }

                             if(chartPointer.labelFn) {
                                 chartPointer.setStyles(Ext.apply({}, chartPointer.extraStyle, chartPointer.chartStyle));
                             }
                    }
                });
            }
        }
    },

    clearSelectedCache: function() {
        this.cacheName = null;
        this.comboCache.clearValue();
        this.chartStore.remove();
        // TODO Add code here to clear chart for old cache.
    },

    updateChartData: function(offset) {

       /* if (Ext.get('cacheEfficiency-tab').isVisible()) {
            //this.loadChartStore(offset);

            // TODO Consider whether these are in the wrong place and should be moved.
            if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null && this.cacheName !== null) {

                if (offset == 'back') {
                    this.live = false;
                }
                if (this.live) {
                    this.forwardButton.disable();
                    this.backButton.enable();
                    //this.generateValues();


                } else {
                    this.forwardButton.enable();
                    this.backButton.enable();
                }


            }
        } */
    },

    updateData: function() {
            if (!this.comboCacheManager.isExpanded()) {
                this.comboCacheManagerStore.load();
            }
            if (!this.comboCache.isExpanded()) {
                this.loadCacheStore();
            }
       
            this.chartStore.remove();
            this.loadChartStore();
    },

    changeActiveCacheManager: function(data) {
        if (this.cacheManagerAddress != data['address'] || this.cacheManagerPort != data['port']) {
            this.cacheManagerAddress = data['address'];
            this.cacheManagerPort = data['port'];
            this.clearSelectedCache();
            this.loadCacheStore();
            //alert(data['displayName']);
            if(data['displayName'] != "All"){
                 this.comboCacheManager.setValue(data['displayName']) ;

            } else{
                this.comboCache.disable();
                 Ext.getCmp('chartPanel').hide();
               // this.settingsButton.disable();
                this.comboCacheManager.setValue(null);

            }


            //ref re-enable
            this.comboCache.enable();

        }
    },

    changeActiveCache: function(data) {
        if (this.cacheName != data['name']) {
            this.cacheName = data['name'];
            this.updateChartData();

            Ext.getCmp('chartPanel').show();
           // Ext.getCmp('settingsButton').enable();
            if(this.comboCacheStore.getCount() == 0){
                        Ext.getCmp('comboCache').disable();
                    }else{
                       Ext.getCmp('comboCache').enable();
                 this.comboCache.setValue(this.formatCacheName(data['name']));
              }
        }
    },

    formatCacheName: function(fullName) {
        // anything shorter than 37 chars will not be shortened
        if (fullName.length < 37) {
            return fullName;
        } else {
            var result = '';

            var comps = fullName.split('.');
            var truncate = true;
            for (var i = 0; i < comps.length; i++) {
                var comp = comps[i];
                if (truncate &&
                    comp.charCodeAt(0) < 97) { // check if it's uppercase by looking at the 'A' ascii code
                    truncate = false;
                }

                if (result.length > 0) {
                    result += '.';
                }

                if (truncate) {
                    result += comp.charAt(0);
                } else {
                    result += comp;
                }
            }

            return result;
        }
    },

    createChartStore: function() {
        this.chartStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            url: '/monitor/getSampledStatisticHistory',
            baseParams: {
                name: this.statType,
                format: 'xml'
            },
            record: 'sampledStatisticHistory > sample',
            fields: [
                // TODO Should the type also be parameterised by stat?
                {name: 'value', mapping: 'value', type: 'float'},
                {name: 'time', mapping: 'timestamp', type: 'date', dateFormat: 'Y/m/d H:i:s'}
            ]
        });
    },

    createStores: function() {
        this.comboCacheManagerStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: true,
            autoDestroy: true,
            sortInfo: {
                field: 'displayName',
                direction: 'ASC'
            },
            url: '/monitor/listProbes',
            baseParams: {
                format: 'xml'
            },
            record: 'probe',
            fields: [ 'address', 'port', 'probeName', 'displayName' ]
        });

        this.comboCacheStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            sortInfo: {
                field: 'name',
                direction: 'ASC'
            },
            baseParams: {
                format: 'xml'
            },
            record: 'name',
            fields: [
                {
                    name: 'name',
                    mapping: '/',
                    type: 'string'
                },
                {
                    name: 'adaptedName',
                    mapping: '/',
                    type: 'string',
                    convert: this.formatCacheName
                }
            ]
        });

        this.createChartStore();
    },

    createChartInterface: function() {
        var tabPanel = this;

        // TODO If live, set maximum to last timestamp when data is received. Set minimum relative to maximum.
        // TODO If not live, set minimum to first timestamp when data is received. Set maximum relative to minimum.

        this.xAxis = new Ext.chart.TimeAxis({
            title: '',
            labelRenderer: function(date) {
                return date.format("H:i:s");
            },
            minimum: this.minimum,
            maximum: this.maximum,
            // TODO This is suitable for a 1 minute time interval, but not the others.
            majorTimeUnit: 'seconds',
            majorUnit: this.timeInterval/10
        });
        this.yAxis = new Ext.chart.NumericAxis({
            title: this.yAxisTitles[this.statType],
            minimum: 0,
            maximum: 100,
            labelRenderer: Ext.util.Format.numberRenderer(this.yAxisFormats[this.statType])
        });

//// TODO If we get problems with reloading data into a chart that is hidden, try:
//
//        var chart = new Ext.chart.LineChart({
//        ...
//        plugins: [new Ext.ux.plugin.VisibilityMode()],
//        ...
//        });
        this.chart = new Ext.chart.LineChart({
            store: this.chartStore,
            id: 'chart',
            xAxis: this.xAxis,
            yAxis: this.yAxis,
            xField: 'time',
            expressInstall: true,
            style: "background: url('/monitor/images/fpr.png'); background-position: center center; background-repeat: no-repeat;",
            yField: 'value',
                // TODO Decide what to display for tips over data points, if anything.
//            tipRenderer : function(chart, record){
//                return Ext.util.Format.number(record.data.visits, '0,0') + ' visits in ' + record.data.name;
//            }
            chartStyle: {
                padding: 10,
                animationEnabled: false,
                font: {
                   name: 'Tahoma',
                   color: 0x444444,
                   size: 11
                },
                dataTip: {
                    padding: 5,
                    border: {
                        color: 0x99bbe8,
                        size:1
                    },
                   background: {
                        color: 0xDAE7F6,
                        alpha: .9
                    }
               }
            },
            series: [{
                type: 'line',
                displayName: '',
                yField: 'value',
                style: {
                            color: 0x72504E,
                            image: '',
                            mode: 'stretch'
                        }
            }],
            extraStyle: {
                xAxis: {
                    labelRotation: 0
                },
                yAxis: {
                    titleRotation: -90
                }
            }
        });

        this.chartPanel = new Ext.Panel({
            tabPanel: this,
            iconCls: 'chart',
            hidden: true,
            id: "chartPanel",
            title: this.chartTitles[this.statType],
            frame: true,
            applyTo: 'cache-efficiency-chart',
            width: 800,
            height: 300,
            layout: 'fit',
            items: this.chart
        });
    },

    // TODO Validate that both date and time are set if Live is unchecked.
    //          Currently doesn't load any chart data if not live and date or time is not set.
    // TODO When From date or time is entered, uncheck Live and enable date and time.
    createSettingsInterface: function() {
        var liveCheckbox = new Ext.form.Checkbox({
            fieldLabel: 'Live',
            boxLabel: '',
            name: 'live',
            checked: true,
            listeners: {check: function (checkbox, checked){
               if(checked){
                   formPanel.findById('fromTime').disable();
                   formPanel.findById('fromDate').disable();
               }   else{
                   formPanel.findById('fromTime').enable();
                   formPanel.findById('fromDate').enable();
               }
            }}
        });

        // TODO Restrict to dates and times in the past - max/minValue attributes.

        var fromDateField = new Ext.form.DateField({
            fieldLabel: 'From',
            id:'fromDate',
            disabled: true,
            name: 'fromDate',
            format: 'Y/m/d'
//            editable: false
        });

        // TODO Move alongside date field.
        // TODO Need a better time picker where we can select hours and minutes separately.
        var fromTimeField = new Ext.form.TimeField({
            name: 'fromTime',
            id: 'fromTime',
            disabled: true,
            increment: 1,
            format: 'H:i'
//            editable: false
//            enableKeyEvents: true
        });

//            var date = new Date();
//            date.setTime(date.getTime()+(days*24*60*60*1000));
//            var exptime = "; expires="+date.toGMTString();

        var tabPanel = this;
        //have the selection of the live and from box to be mutually exclusive
       /* var formPanel = new Ext.form.FormPanel({
            tabPanel: this,
            id: 'formPanel',
            labelWidth: 60,
            frame: false,
            bodyStyle: 'padding: 5px 5px 0',
            width: 500,
            autoHeight: true,
            items: [
            // TODO Reinstate control to choose chart type.
               /* new Ext.form.ComboBox({
                    name: 'statType',
                    fieldLabel: 'Type',
                    width: 360,
                    mode: 'local',
                    triggerAction: 'all',
                    selectOnFocus: false,
                    forceSelection: true,
                    typeAhead: true,
                    triggerClass: 'monitor-combo-trigger-360',
                    store: tabPanel.statTypeStore,
                    value: tabPanel.statType
                }),*/
                /*liveCheckbox,
                fromDateField,
                fromTimeField,  */
              /*  new Ext.form.ComboBox({
                    name: 'timeInterval',
                    fieldLabel: 'Time Interval',
//            editable: false
                    width: 360,
                    mode: 'local',
                    triggerAction: 'all',
                    selectOnFocus: false,
                    forceSelection: true,
                    typeAhead: true,
                    triggerClass: 'monitor-combo-trigger-360',
                    // TODO Leave out options less than the configured ehcache sample rate (default is 10 seconds).
                    //      Will need to query the server. See code in monitor.js.
                    // TODO Add sample rate in here as the minimum value.
                    store: [[100, '10 Seconds'],[600, '1 minute'], [3000, '5 minutes'], [18000, '30 minutes'], [36000, '1 hour'], [864000, '1 day']],
                    value: 100

                })
            ],
            // TODO Add normal shortcut keys for these buttons - Enter and Escape.
            buttons: [
                {
                    text: 'OK',
                    scope: this,
                    tabPanel: this,
                    handler: function() {
                        var values = Ext.getCmp('formPanel').getForm().getFieldValues();
                        this.statType = values['statType'];
                        this.timeInterval = values['timeInterval'];
                        this.settingsWindow.hide();
                        this.monitor.updateCacheEfficiencyTab();
                    }
                },
                {
                    text: 'Cancel',
                    scope: this,
                    handler: function() {
                        this.settingsWindow.hide();
                    }
                }
            ]
        });

        this.settingsWindow = new Ext.Window({
            tabPanel: this,
            animCollapse: true,
            animateTarget: 'cache-efficiency-settings',
            title: 'Chart Settings',
            resizable: false,
            layout: 'fit',
            autoWidth: true,
            autoHeight: true,
            closable: false,
            modal: true,
            shim: true,
            buttonAlign: 'center',
            closeAction: 'hide',
            items: [
                formPanel
            ]
        });

        this.settingsButton = new Ext.Button({
            scope: this,
            id: 'settingsButton',
            disabled: true,
            renderTo: 'cache-efficiency-settings',
            text: 'Chart Settings',
            handler: function() {
                formPanel.getForm().setValues({
                    statType: this.statType,
                    //live: this.live,
                    //fromDate: this.fromDate,
                    //fromTime: this.fromTime,
                    timeInterval: this.timeInterval
                });
                this.settingsWindow.show('cache-efficiency-settings');
            }
        });*/
    },

    createInterface: function() {
        this.createChartInterface();
        this.createSettingsInterface();

        this.comboCacheManager = new TC.components.ComboBox({
            tabPanel: this,
            store: this.comboCacheManagerStore,
            displayField: 'displayName',
            editable: false,
            emptyText: 'Select a cache manager...',
            applyTo: 'efficiency-cache-manager',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCacheManager(record.data);
                     if(this.getStore().getCount() == 0){
                        Ext.getCmp('comboCache').disable();
                    }else{
                       Ext.getCmp('comboCache').enable();
                    }
                    Ext.getCmp('chartPanel').hide();
                }
            }
        });

        this.comboCache = new TC.components.ComboBox({
            tabPanel: this,
            store: this.comboCacheStore,
            displayField: 'adaptedName',
            emptyText: 'Select a cache...',
            editable: false,
            id: 'comboCache',
            disabled: true,
            applyTo: 'efficiency-cache',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCache(record.data);
                    this.tabPanel.monitor.updateCacheEfficiencyTab();
                    Ext.getCmp('chartPanel').show();
                }
            }
        });

        //COMBO FOR CHARTTYPE
        this.chartTypeCombo = new TC.components.ComboBox({
                    name: 'statType',
                    fieldLabel: 'Type',
                    width: 360,
                    tabPanel: this,
                    mode: 'local',
                    editable: false,
                    triggerAction: 'all',
                    selectOnFocus: false,
                    forceSelection: true,
                    typeAhead: true,
                    applyTo: 'chart-selector-combo',
                    triggerClass: 'monitor-combo-trigger-360',
                    store: this.statTypeStore,
                    value: this.statType,
                    scope: this,
                    listeners: {
                        'select': function(combo, record) {
                            this.tabPanel.statType = this.getValue();
                            this.tabPanel.monitor.updateCacheEfficiencyTab();
                        }
                    }
                });

        // TODO Replace text with icons.
        this.forwardButton = new Ext.Button({
            tabPanel: this,
           // renderTo: 'cache-efficiency-forward',
            text: 'Forward',
            handler: function() {
                //this.tabPanel.updateChartData('forward');
            }
        });
        this.forwardButton.disable();

        this.backButton = new Ext.Button({
            tabPanel: this,
            //renderTo: 'cache-efficiency-back',
            text: 'Back',
            handler: function() {
                //this.tabPanel.updateChartData('back');
            }
        });
    },

    generateValues: function(){
        var maximum = new Date();
        var date=new Date();
        if(this.loadFlag){
            if(this.chartStore.getCount() > 0){
                maximum = new Date(this.chartStore.getAt(this.chartStore.getCount()-1).data.time);
                date = new Date(this.chartStore.getAt(this.chartStore.getCount()-1).data.time);
            }
        }

        var minutes=date.getMinutes();
        var seconds=date.getSeconds();
        var milliseconds=date.getMilliseconds();
        date.setMinutes(minutes,seconds-(this.timeInterval*0.9),milliseconds);
        var HH=date.getHours();
        var MM=date.getMinutes();
        HH<10?HH='0'+HH:null;
        MM<10?MM='0'+MM:null;
        var minimum =date;

        this.maximum = maximum;
        this.minimum = minimum;
    }
};
