/*
 * All content copyright (c) 2003-2009 Terracotta, Inc., except as may otherwise be noted in a separate copyright notice.  All rights reserved.
 */
Ext.ns('TC.monitor.tabs');

TC.monitor.tabs.CacheManagers = function(monitor) {
    this.monitor = monitor;

    this.createStores();

    this.createInterface();

};

TC.monitor.tabs.CacheManagers.prototype = {
    monitor: null,

    table: null,

    tableStore: null,

    updateData: function() {
        if (Ext.get('cacheManagers-tab').isVisible()) {
            Ext.get("registered-cache-managers").load({
                url: '/monitor/countProbes'
            });
            if(this.tableStore.getCount() > 0){
                this.tableStore.remove();
            }
            this.tableStore.load();
        }
    },

    formatProbe: function(probeName, record) {
        if (probeName != '') {
            return probeName;
        } else {
            return record.getElementsByTagName("address")[0].childNodes[0].nodeValue + ':' + record.getElementsByTagName("port")[0].childNodes[0].nodeValue;
        }
    },

    createStores: function() {
        this.tableStore = new Ext.data.XmlStore({
            autoLoad: false,
            autoDestroy: true,
            sortInfo: {
                field: 'probe',
                direction: 'ASC'
            },
            url: '/monitor/getAggregatedSampledStatisticValuesPerProbe',
            baseParams: {
                name: ['hit', 'miss', 'put', 'totalCount'],
                format: 'xml'
            },
            record: 'probe',
            fields: [
                {
                    name: 'displayName',
                    type: 'string'
                },
                {
                    name: 'probeName',
                    type: 'string'
                },
                {
                    name: 'address',
                    type: 'string'
                },
                {
                    name: 'port',
                    type: 'string'
                },
                {
                    name: 'probe',
                    mapping: 'probeName',
                    type: 'string',
                    convert: this.formatProbe
                },
                {
                    name: 'cacheManagerName',
                    type: 'string'
                },
                {
                    name: 'replicationMode',
                    type: 'string'
                },
                {
                    name: 'cacheCount',
                    mapping: 'sampledStatisticValues > cacheCount',
                    type: 'int'
                },
                {
                    name: 'totalCount',
                    mapping: 'sampledStatisticValues > totalCount',
                    type: 'int'
                },
                {
                    name: 'hit',
                    mapping: 'sampledStatisticValues > hit',
                    type: 'float'
                },
                {
                    name: 'miss',
                    mapping: 'sampledStatisticValues > miss',
                    type: 'float'
                },
                {
                    name: 'put',
                    mapping: 'sampledStatisticValues > put',
                    type: 'float'
                }
            ]
        });
    },
    
    createInterface: function() {

        this.table = new Ext.grid.GridPanel({
            tabPanel: this,
            store: this.tableStore,
            renderTo: 'cache-managers-table',
           stateId: 'ehcachedx-cache-managers-table',
            stateful: true,
            autoExpandColumn: 'probe',
            height: 500,
            autoHeight: true,
            listeners: {
                'cellclick': function(grid, row) {
                    var record = grid.getStore().getAt(row);
                    this.tabPanel.monitor.changeActiveCacheManager(record.data);
                    this.tabPanel.monitor.showStatisticsTab();
                }
            },
            colModel: new Ext.grid.ColumnModel({
                defaults: {
                    width: 90,
                    sortable: true,
                    xtype: 'numbercolumn',
                    format: '0,000',
                    align: 'right'
                },
                columns: [
                    {
                        id: 'probe',
                        header: "Probe",
                        dataIndex: 'probe',
                        hideable: false,
                        xtype: 'gridcolumn',
                        align: 'left',
                        tooltip: 'The name of the probe or it\'s address and port.'
                    },
                    {
                        header: "Cache Manager",
                        dataIndex: 'cacheManagerName',
                        hideable: false,
                        xtype: 'gridcolumn',
                        align: 'left',
                        tooltip: 'The name of the cache manager.',
                        width: 140
                    },
                    {
                        header: 'Replication',
                        dataIndex: 'replicationMode',
                        xtype: 'gridcolumn',
                        align: 'left',
                        tooltip: 'The way the caches in this cache manager are replicated, possible values are: None, RMI, JGroups, JMS, Terracotta and Custom when a replication mode is used.'
                    },
                    {
                        header: 'Caches',
                        dataIndex: 'cacheCount',
                        tooltip: 'The number of caches in the cache manager.'
                    },
                    {
                        header: 'Total Elements',
                        dataIndex: 'totalCount',
                        width: 100,
                        tooltip: 'The number of elements in all the caches of this cache manager.'
                    },
                    {
                        header: 'Hits/s',
                        dataIndex: 'hit',
                        format: '0,000.0',
                        tooltip: 'The number of times a requested item was found across all the caches in this cache manager.'
                    },
                    {
                        header: 'Misses/s',
                        dataIndex: 'miss',
                        format: '0,000.0',
                        tooltip: 'The number of times a requested item was not found across all the caches in this cache manager.'
                    },
                    {
                        header: 'Puts/s',
                        dataIndex: 'put',
                        format: '0,000.0',
                        tooltip: 'The number of elements that were put across all the caches in this cache manager.'
                    }
                ]
            })
        });
      
    }
};
