/*
 * All content copyright (c) 2003-2009 Terracotta, Inc., except as may otherwise be noted in a separate copyright notice.  All rights reserved.
 */
Ext.ns('TC.monitor.tabs');

TC.monitor.tabs.Configuration = function(monitor) {
    this.monitor = monitor;
    this.createStores();
    this.createInterface();
};

TC.monitor.tabs.Configuration.prototype = {
    monitor: null,

    cacheManagerAddress: null,
    cacheManagerPort: null,

    comboCacheManager: null,
    table: null,
    tableAll: null,

    comboCacheManagerStore: null,
    tableStore: null,
    tableAllStore: null,

    // TODO The scroll bar resets to the top every time data is loaded here.
    loadTableStore: function() {
        if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null) {
            this.tableStore.remove();
            this.tableStore.load({
                params: {
                    'address': this.cacheManagerAddress,
                    'port': this.cacheManagerPort
                }
            });
        } else {
            this.tableAllStore.load();
        }
    },

    updateData: function() {
        if (Ext.get('configuration-tab').isVisible()) {
            if (!this.comboCacheManager.isExpanded()) {
                this.comboCacheManagerStore.load();
            }
            this.loadTableStore();
        }
    },

    changeActiveCacheManager: function(data) {
        if (this.cacheManagerAddress != data['address'] || this.cacheManagerPort != data['port']) {
            this.cacheManagerAddress = data['address'];
            this.cacheManagerPort = data['port'];
            this.loadTableStore();
            this.comboCacheManager.setValue(data['displayName'])
        }
    },

    createStores: function() {
        this.comboCacheManagerStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: true,
            autoDestroy: true,
            sortInfo: {
                field: 'displayName',
                direction: 'ASC'
            },
            url: '/monitor/listProbes',
            baseParams: {
                format: 'xml'
            },
            record: 'probe',
            fields: [ 'address', 'port', 'probeName', 'displayName' ],
            listeners: {
                'load': function(store) {
                    store.insert(0, new store.recordType({
                        displayName: 'All',
                        name: '',
                        address: null,
                        port: null
                    }));
                }
            }
        });

        this.tableStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            sortInfo: {
                field: 'name',
                direction: 'ASC'
            },
            url: '/monitor/getAllCachesConfigAllValues',
            baseParams: {
                format: 'xml'
            },
            record: 'cache',
            fields: [
                {
                    name: 'name',
                    type: 'string'
                },
                {
                    name: 'statistics',
                    mapping: 'configValues > statistics',
                    type: 'string'
                },
                {
                    name: 'overflowToOffHeap',
                    mapping: 'configValues > overflowToOffHeap',
                    type: 'string'
                },
                {
                    name: 'maxMemoryOffHeap',
                    mapping: 'configValues > maxMemoryOffHeap',
                    type: 'string'
                },
                {
                    name: 'diskExpiryThreadIntervalSeconds',
                    mapping: 'configValues > diskExpiryThreadIntervalSeconds',
                    type: 'int'
                },
                {
                    name: 'diskPersistent',
                    mapping: 'configValues > diskPersistent',
                    type: 'boolean'
                },
                {
                    name: 'diskSpoolBufferSizeMB',
                    mapping: 'configValues > diskSpoolBufferSizeMB',
                    type: 'int'
                },
                {
                    name: 'eternal',
                    mapping: 'configValues > eternal',
                    type: 'boolean'
                },
                {
                    name: 'evictionPolicy',
                    mapping: 'configValues > evictionPolicy',
                    type: 'string'
                },
                {
                    name: 'maxElementsInMemory',
                    mapping: 'configValues > maxElementsInMemory',
                    type: 'int'
                },
                {
                    name: 'maxElementsOnDisk',
                    mapping: 'configValues > maxElementsOnDisk',
                    type: 'int'
                },
                {
                    name: 'overflowToDisk',
                    mapping: 'configValues > overflowToDisk',
                    type: 'boolean'
                },
                {
                    name: 'timeToIdleSeconds',
                    mapping: 'configValues > timeToIdleSeconds',
                    type: 'int'
                },
                {
                    name: 'timeToLiveSeconds',
                    mapping: 'configValues > timeToLiveSeconds',
                    type: 'int'
                }
            ],
            listeners: {
                'beforeload': function(store) {
                    if (this.tabPanel.table.hidden) {
                        store.removeAll();
                        this.tabPanel.tableAll.setVisible(false);
                        this.tabPanel.table.setVisible(true);
                    }
                }
            }
        });

        var tableAllReader = new Ext.data.XmlReader({
            record: 'cache',
            fields: [
                {
                    name: 'name',
                    mapping: 'name',
                    type: 'string'
                },
                {
                    name: 'probe',
                    mapping: 'probe > displayName',
                    type: 'string'
                },
                {
                    name: 'statistics',
                    mapping: 'probe > configValues > statistics',
                    type: 'string'
                },
                {
                    name: 'overflowToOffHeap',
                    mapping: 'probe > configValues > overflowToOffHeap',
                    type: 'boolean'
                },
                {
                    name: 'maxMemoryOffHeap',
                    mapping: 'probe > configValues > maxMemoryOffHeap',
                    type: 'string'
                },
                {
                    name: 'diskExpiryThreadIntervalSeconds',
                    mapping: 'probe > configValues > diskExpiryThreadIntervalSeconds',
                    type: 'int'
                },
                {
                    name: 'diskPersistent',
                    mapping: 'probe > configValues > diskPersistent',
                    type: 'boolean'
                },
                {
                    name: 'diskSpoolBufferSizeMB',
                    mapping: 'probe > configValues > diskSpoolBufferSizeMB',
                    type: 'int'
                },
                {
                    name: 'eternal',
                    mapping: 'probe > configValues > eternal',
                    type: 'boolean'
                },
                {
                    name: 'evictionPolicy',
                    mapping: 'probe > configValues > evictionPolicy',
                    type: 'string'
                },
                {
                    name: 'maxElementsInMemory',
                    mapping: 'probe > configValues > maxElementsInMemory',
                    type: 'int'
                },
                {
                    name: 'maxElementsOnDisk',
                    mapping: 'probe > configValues > maxElementsOnDisk',
                    type: 'int'
                },
                {
                    name: 'overflowToDisk',
                    mapping: 'probe > configValues > overflowToDisk',
                    type: 'boolean'
                },
                {
                    name: 'timeToIdleSeconds',
                    mapping: 'probe > configValues > timeToIdleSeconds',
                    type: 'int'
                },
                {
                    name: 'timeToLiveSeconds',
                    mapping: 'probe > configValues > timeToLiveSeconds',
                    type: 'int'
                }
            ]
        });
        this.tableAllStore = new Ext.data.GroupingStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            sortInfo: {
                field: 'probe',
                direction: 'ASC'
            },
            groupField: 'name',
            url: '/monitor/getAggregatedConfigValuesPerCache',
            baseParams: {
                format: 'xml'
            },
            reader: tableAllReader,
            listeners: {
                'beforeload': function(store) {
                    if (this.tabPanel.tableAll.hidden) {
                        store.removeAll();
                        this.tabPanel.tableAll.setVisible(true);
                        this.tabPanel.table.setVisible(false);
                    }
                }
            }
        });
    },

    addConfigurationColumns: function(columns) {
        columns.push({
            header: 'Statistics',
            dataIndex: 'statistics',
            xtype: 'gridcolumn',
            align: 'left',
            tooltip: 'Indicates whether cache statistics gathering is enabled.'
        },
		{
            header: 'OffHeap',
            dataIndex: 'overflowToOffHeap',
            xtype: 'gridcolumn',
            align: 'left',
            tooltip: 'Indicates whether cache is using an OffHeap store.'
        },
		{
            header: 'Max OffHeap',
            dataIndex: 'maxMemoryOffHeap',
            xtype: 'gridcolumn',
            tooltip: 'The maximum size of the OffHeap store (bytes).'
        },
        {
            header: 'Eternal',
            dataIndex: 'eternal',
            xtype: 'booleancolumn',
            align: 'left',
            tooltip: 'Indicates whether elements are eternal. If eternal, timeouts are ignored and the ' +
                     'element is never expired.'
        },
        {
            header: 'TTI',
            dataIndex: 'timeToIdleSeconds',
            tooltip: 'The time to idle for an element before it expires, i.e. the maximum amount of time ' +
                     'between accesses before an element expires. Is only used if the element is not ' +
                     'eternal. A value of 0 means that an Element can idle for infinity.'
        },
        {
            header: 'TTL',
            dataIndex: 'timeToLiveSeconds',
            tooltip: 'The time to live for an element before it expires, i.e. the maximum time between ' +
                     'creation time and when an element expires. Is only used if the element is not ' +
                     'eternal. A value of 0 means that and Element can live for infinity.'
        },
        {
            header: 'Max Mem',
            dataIndex: 'maxElementsInMemory',
            tooltip: 'The maximum number of objects that are created in memory.'
        },
        {
            header: 'Eviction',
            dataIndex: 'evictionPolicy',
            xtype: 'gridcolumn',
            align: 'left',
            tooltip: 'Policy would be enforced upon reaching the maxElementsInMemory limit. Default policy ' +
                     'is Least Recently Used (specified as LRU). Other policies available - First In First ' +
                     'Out (specified as FIFO) and Less Frequently Used (specified as LFU)'
        },
        {
            header: 'Overflow to Disk',
            dataIndex: 'overflowToDisk',
            width: 120,
            xtype: 'booleancolumn',
            align: 'left',
            tooltip: 'Indicates whether elements can overflow to disk when the memory store has reached the ' +
                     'maxInMemory limit.'
        },
        {
            header: 'Max Disk',
            dataIndex: 'maxElementsOnDisk',
            tooltip: 'The maximum number of objects that are maintained in the DiskStore.'
        },
        {
            header: 'Persistent',
            dataIndex: 'diskPersistent',
            xtype: 'booleancolumn',
            align: 'left',
            tooltip: 'Indicates whether the disk store persists between restarts of the Virtual Machine.'
        },
        // TODO These are never shown at the moment.
        {
            header: 'Disk Buffer Size',
            dataIndex: 'diskSpoolBufferSizeMB',
            width: 110,
            hidden: true,
            format: '0,000MB',
            tooltip: 'This is the size to allocate the DiskStore for a spool buffer. Writes are made to ' +
                     'this area and then asynchronously written to disk. Each spool buffer is used only by ' +
                     'its cache. If you get OutOfMemory errors consider lowering this value. To improve ' +
                     'DiskStore performance consider increasing it. Trace level logging in the DiskStore ' +
                     'will show if put back ups are occurring.'
        },
        {
            header: 'Disk Expiry Interval',
            dataIndex: 'diskExpiryThreadIntervalSeconds',
            width: 110,
            hidden: true,
            format: '0,000s',
            tooltip: 'The number of seconds between runs of the disk expiry thread.'
        });
        return columns;
    },

    createInterface: function() {
        this.comboCacheManager = new TC.components.ComboBox({
            tabPanel: this,
            store: this.comboCacheManagerStore,
            displayField: 'displayName',
            emptyText: 'Select a cache manager...',
            applyTo: 'configuration-cache-manager',
            value: 'All',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCacheManager(record.data);
                }
            }
        });

        

        this.table = new Ext.grid.GridPanel({
            tabPanel: this,
            store: this.tableStore,
            renderTo: 'configuration-table',
            stateId: 'ehcachedx-configuration-table',
            stateful: true,
            autoExpandColumn: 'name',
            forceFit: true,
            height: 500,
            autoHeight: true,
            listeners: {
                'cellclick': function(grid, row) {
                    var record = grid.getStore().getAt(row);
                    this.tabPanel.monitor.changeActiveCache(record.data);
                    this.tabPanel.monitor.showContentsTab();
                }
            },
            hidden: true,
            colModel: new Ext.grid.ColumnModel({
                defaults: {
                    width: 80,
                    sortable: true,
                    xtype: 'numbercolumn',
                    format: '0,000',
                    align: 'right'
                },
                columns: this.addConfigurationColumns([
                    {
                        id: 'name',
                        header: "Cache",
                        dataIndex: 'name',
                        hideable: false,
                        xtype: 'gridcolumn',
                        align: 'left',
                        tooltip: 'The name of the cache'
                    }

                ])
            })
        });

        this.tableAll = new Ext.grid.GridPanel({
            tabPanel: this,
            store: this.tableAllStore,
            renderTo: 'configuration-all-table',
            stateId: 'ehcachedx-configuration-all-table',
            stateful: true,
            autoExpandColumn: 'probe',
            height: 500,
            autoHeight: true,
            view: new Ext.grid.GroupingView({

                autoFill: true,
                enableGroupingMenu: false,
                enableNoGroups: false,
                hideGroupedColumn: true,
                startCollapsed: true,
                groupTextTpl: '{text} <span style="font-weight: normal;">({[values.rs.length]} {[values.rs.length > 1 ? "cache managers" : "cache manager"]})</span>'
            }),            
            listeners: {
                'cellclick': function(grid, row) {
                    var record = grid.getStore().getAt(row);
                    this.tabPanel.monitor.changeActiveCache(record.data);
                    this.tabPanel.monitor.showContentsTab();
                }
            },
            colModel: new Ext.grid.ColumnModel({
                defaults: {
                    width: 80,
                    sortable: true,
                    xtype: 'numbercolumn',
                    format: '0,000',
                    align: 'right'
                },
                columns: this.addConfigurationColumns([
                    {
                        id: 'probe',
                        header: "Probe",
                        dataIndex: 'probe',
                        hideable: false,
                        xtype: 'gridcolumn',
                        align: 'left',
                        width: 300
                    },
                    {
                        id: 'name',
                        header: "Cache",
                        dataIndex: 'name',
                        xtype: 'gridcolumn',
                        align: 'left'
                    }
                ])
            })
        });
    }
};
