/*
 * All content copyright (c) 2003-2009 Terracotta, Inc., except as may otherwise be noted in a separate copyright notice.  All rights reserved.
 */
Ext.ns('TC.monitor.tabs');

TC.monitor.tabs.Contents = function(monitor) {
    this.monitor = monitor;
    this.createStores();
    this.createInterface();
};

TC.monitor.tabs.Contents.prototype = {
    monitor: null,

    cacheManagerAddress: null,
    cacheManagerPort: null,
    cacheName: null,

    comboCacheManager: null,
    comboCache: null,
    panelSummary: null,
    removeAllButton: null,
    panelSearch: null,
    inputSearch: null,
    tableSearch: null,
    tableSearchAll: null,
    removeSearchButton: null,
    panelDetails: null,
    removeEntryButton: null,

    comboCacheManagerStore: null,
    comboCacheStore: null,
    summaryStore: null,
    tableSearchStore: null,
    tableSearchAllStore: null,
    detailsStore: null,

    loadCacheStore: function() {
        if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null) {
           if( this.comboCacheStore.getCount() > 0){
                 this.comboCacheStore.remove();
            }
            this.comboCacheStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getCacheNames'});
            this.comboCacheStore.load({
                params: {
                    'address': this.cacheManagerAddress,
                    'port': this.cacheManagerPort
                }
            });
        } else {
            this.comboCacheStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getAggregatedCacheNames'});
            this.comboCacheStore.load();
        }
    },

    loadSummaryStore: function() {
        if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null && this.cacheName !== null) {
            if( this.summaryStore.getCount() > 0){
                 this.summaryStore.remove();
            }
            this.summaryStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getSampledStatisticValues'});
            this.summaryStore.load({
                params: {
                    'address': this.cacheManagerAddress,
                    'port': this.cacheManagerPort,
                    'cache': this.cacheName
                }
            });
        } else if (this.cacheName !== null) {
            this.summaryStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getAggregatedSampledStatisticValuesPerCache'});
            this.summaryStore.load({
                params: {
                    'cache': this.cacheName
                }
            });
        }
    },

    loadSearchStore: function() {
        var query = this.inputSearch.getValue();
        var params = {
            'cache': this.cacheName
        };
        if (query != '') {
            params['query'] = query;
        }
        if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null && this.cacheName !== null) {
            params['address'] = this.cacheManagerAddress;
            params['port'] = this.cacheManagerPort;
            this.tableSearchStore.load({
                params: params
            });
        } else if (this.cacheName !== null) {
            this.tableSearchAllStore.load({
                params: params
            });
        }
    },

    loadDetailsStore: function(data) {
        var address = this.cacheManagerAddress;
        var port = this.cacheManagerPort;

        if (!Ext.isEmpty(data['address'])) {
            address = data['address'];
        }
        if (!Ext.isEmpty(data['port'])) {
            port = data['port'];
        }

        if (address !== null && port !== null &&
            this.cacheName !== null && Ext.isDefined(data['key']) && data['key'] != null) {
            this.detailsStore.load({
                params: {
                    'address': address,
                    'port': port,
                    'cache': this.cacheName,
                    'key': data['key']
                },
                scope: data['key']
            });
        }
    },

    updateSummaryCounts: function(data) {
        for (var i in data) {
            var element = Ext.get('contents-summary-' + i);
            if (element !== null) {
                element.update(data[i]);
            }
        }
    },

    updateSearchResultCount: function(resultCount) {
        Ext.get('contents-search-results').show();
        if (resultCount == 0) {
            this.removeSearchButton.disable();
        } else {
            var query = this.inputSearch.getValue();
            if (Ext.isEmpty(query)) {
                this.removeSearchButton.disable();
            } else {
                this.removeSearchButton.enable();
            }
            
            if (resultCount >= 100) {
                resultCount = ">=" + resultCount;
            }
        }

        Ext.get('contents-search-count').update("" + resultCount);
    },

    updateDetails: function(data) {
        if (Ext.isEmpty(data['created'])) {
            this.removeEntryButton.disable();

            Ext.iterate(Ext.query('tr[class=content-details-match]'), function(item) {
                var el = Ext.get(item);
                el.setVisibilityMode(Ext.Element.DISPLAY);
                el.hide();
            });
            Ext.iterate(Ext.query('tr[class=content-details-nomatch]'), function(item) {
                var el = Ext.get(item);
                el.setVisibilityMode(Ext.Element.DISPLAY);
                el.show();
            });
        } else {
            this.removeEntryButton.enable();

            Ext.iterate(Ext.query('tr[class=content-details-match]'), function(item) {
                var el = Ext.get(item);
                el.setVisibilityMode(Ext.Element.DISPLAY);
                el.show();
            });
            Ext.iterate(Ext.query('tr[class=content-details-nomatch]'), function(item) {
                var el = Ext.get(item);
                el.setVisibilityMode(Ext.Element.DISPLAY);
                el.hide();
            });
        }

        for (var i in data) {
            var element = Ext.get('contents-details-' + i);
            if (element !== null) {
                element.update(data[i]);
            }
        }
    },

    clearSelectedCache: function() {
        this.cacheName = null;
        this.comboCache.clearValue();
        this.panelSummary.collapse();
        this.removeAllButton.disable();

        this.updateSummaryCounts({'totalCount': '?', 'memoryCount': '?', 'diskCount': '?'});

        this.panelSearch.collapse();
        this.inputSearch.disable();
        this.tableSearch.disable();
        this.tableSearchStore.removeAll();
        this.tableSearchAll.disable();
        this.tableSearchAllStore.removeAll();
        Ext.get('contents-search-results').hide();

        this.removeSearchButton.disable();
        this.panelDetails.collapse();
    },

    updateData: function() {
        if (Ext.get('contents-tab').isVisible()) {
            if (!this.comboCacheManager.isExpanded()) {
                this.comboCacheManagerStore.load();
            }
            if (!this.comboCache.isExpanded()) {
                this.loadCacheStore();
            }
            this.loadSummaryStore();
        }
    },

    changeActiveCacheManager: function(data) {
        if (this.cacheManagerAddress != data['address'] || this.cacheManagerPort != data['port']) {
            this.cacheManagerAddress = data['address'];
            this.cacheManagerPort = data['port'];
            this.clearSelectedCache();
            this.loadCacheStore();
            this.comboCacheManager.setValue(data['displayName']);
        }
    },

    changeActiveCache: function(data) {
        if (this.cacheName != data['name']) {
            this.cacheName = data['name'];
            this.loadSummaryStore();
            this.loadSearchStore();
             if(this.comboCacheStore.getCount() == 0){
                        Ext.getCmp('comboCache').disable();
                    }else{
                       Ext.getCmp('comboCache').enable();
                 this.comboCache.setValue(this.formatCacheName(data['name']));
              }

        }
    },

    formatCacheName: function(fullName) {
        // anything shorter than 37 chars will not be shortened
        if (fullName.length < 37) {
            return fullName;
        } else {
            var result = '';

            var comps = fullName.split('.');
            var truncate = true;
            for (var i = 0; i < comps.length; i++) {
                var comp = comps[i];
                if (truncate &&
                    comp.charCodeAt(0) < 97) { // check if it's uppercase by looking at the 'A' ascii code
                    truncate = false;
                }

                if (result.length > 0) {
                    result += '.';
                }

                if (truncate) {
                    result += comp.charAt(0);
                } else {
                    result += comp;
                }
            }

            return result;
        }
    },

    removeAll: function() {
        // create a local copy of the variables to ensure that the same data is used as the ones the user confirmed
        var address = this.cacheManagerAddress;
        var port = this.cacheManagerPort;
        var name = this.cacheName;
        var tabPanel = this;

        var removeFromAllCacheManagers = (null == address && null == port);

        var title = "Remove all content from cache '" + name + "'";
        if (removeFromAllCacheManagers) {
            title += " from all cache managers"
        }
        var msg = "Are you sure you want to remove all the contents from<br />cache '" + name + "'";
        if (removeFromAllCacheManagers) {
            msg += " <b>from all cache managers</b>"
        }
        msg += "?<br /><br />This operation is permanent and can't be reverted.";

        Ext.Msg.confirm(title, msg, function(btn) {
            if ('yes' == btn) {
                if (name !== null) {
                    var params = { 'cache': name };
                    if (address !== null && port !== null) {
                        params['address'] = address;
                        params['port'] = port;
                    }
                    Ext.Ajax.request({
                        url: '/monitor/removeAllFromCache',
                        params: params,
                        failure: function() {
                            Ext.Msg.show({
                                title: "Error while removing all from cache '" + name + "'",
                                msg: "An unexpected error occurred while removing all elements<br />from the " +
                                     "cache '" + name + "'.",
                                autoWidth: true,
                                closable: true,
                                buttons: Ext.Msg.OK,
                                icon: Ext.MessageBox.ERROR
                            });
                        },
                        success: function() {
                            Ext.Msg.show({
                                title: "Successful removal",
                                msg: "All elements were successfully removed from the cache '" + name + "'.",
                                autoWidth: true,
                                closable: true,
                                buttons: Ext.Msg.OK,
                                icon: Ext.MessageBox.INFO
                            });

                            tabPanel.loadSearchStore();
                        }
                    });

                }
            }
        });
    },

    removeQuery: function(query) {
        // create a local copy of the variables to ensure that the same data is used as the ones the user confirmed
        var address = this.cacheManagerAddress;
        var port = this.cacheManagerPort;
        var name = this.cacheName;
        var tabPanel = this;

        var removeFromAllCacheManagers = (null == address && null == port);

        var title = "Remove the elements for query '" + query + "' from cache '" + name + "'";
        if (removeFromAllCacheManagers) {
            title += " from all cache managers"
        }
        var msg = "Are you sure you want to remove the elements for query '" + query + "' from<br />cache '" + name + "'";
        if (removeFromAllCacheManagers) {
            msg += " <b>from all cache managers</b>"
        }
        msg += "?<br /><br />This operation is permanent and can't be reverted.<br /><br />The actual elements that " +
               "will be removed are the result of a new query and it's possible that these don't correspond to the " +
               "results that are now visible on screen.";

        Ext.Msg.confirm(title, msg, function(btn) {
            if ('yes' == btn) {
                if (name !== null) {
                    var params = {
                        'format': 'xml',
                        'query': query,
                        'cache': name
                    };
                    if (address !== null && port !== null) {
                        params['address'] = address;
                        params['port'] = port;
                    }
                    Ext.Ajax.request({
                        url: '/monitor/removeQueryFromCache',
                        params: params,
                        failure: function() {
                            Ext.Msg.show({
                                title: "Error while removing the elements for query '" + query + "' from cache '" + name + "'",
                                msg: "An unexpected error occurred while removing the elements for " +
                                     "query '" + query + "'<br />from the cache '" + name + "'.",
                                autoWidth: true,
                                closable: true,
                                buttons: Ext.Msg.OK,
                                icon: Ext.MessageBox.ERROR
                            });
                        },
                        success: function(response) {
                            var count = 0;
                            var responses = response.responseXML.documentElement.getElementsByTagName("removeQueryFromCache");
                            for (var i = 0; i < responses.length; i++) {
                                if (!Ext.isEmpty(responses[i].firstChild)) {
                                    count += Ext.num(responses[i].firstChild.data, 0);
                                }
                            }
                            Ext.Msg.show({
                                title: "Successful removal",
                                msg: count + " elements were successfully removed from the cache '" + name + "'.",
                                autoWidth: true,
                                closable: true,
                                buttons: Ext.Msg.OK,
                                icon: Ext.MessageBox.INFO
                            });

                            tabPanel.loadSearchStore();
                        }
                    });

                }
            }
        });
    },

    removeOne: function(data) {
        // create a local copy of the variables to ensure that the same data is used as the ones the user confirmed
        var address = data['address'];
        var port = data['port'];
        var name = data['cache'];
        var key = data['key'];
        var tabPanel = this;
        Ext.Msg.confirm("Remove the element with key '" + key + "' from cache '" + name + "'",
                "Are you sure you want to remove the element with key '" + key + "' from<br />cache '" + name + "'?" +
                "<br /><br />This operation is permanent and can't be reverted.", function(btn) {
            if ('yes' == btn) {
                // just making sure that the parameters are set and not null, which
                // would cause everything to be removed from all cache managers
                if (address !== null && port !== null && name !== null) {
                    Ext.Ajax.request({
                        url: '/monitor/removeFromCache',
                        params: {
                            'address': address,
                            'port': port,
                            'cache': name,
                            'key': key
                        },
                        failure: function() {
                            Ext.Msg.show({
                                title: "Error while removing the element with key '" + key + "' from cache '" + name + "'",
                                msg: "An unexpected error occurred while removing the element with " +
                                     "key '" + key + "'<br />from the cache '" + name + "'.",
                                autoWidth: true,
                                closable: true,
                                buttons: Ext.Msg.OK,
                                icon: Ext.MessageBox.ERROR
                            });
                        },
                        success: function() {
                            Ext.Msg.show({
                                title: "Successful removal",
                                msg: "The element with key '" + key + "' was successfully removed from the " +
                                     "cache '" + name + "'.",
                                autoWidth: true,
                                closable: true,
                                buttons: Ext.Msg.OK,
                                icon: Ext.MessageBox.INFO
                            });

                            tabPanel.loadSearchStore();
                        }
                    });

                }
            }
        });
    },

    createStores: function() {
        this.comboCacheManagerStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: true,
            autoDestroy: true,
            sortInfo: {
                field: 'displayName',
                direction: 'ASC'
            },
            url: '/monitor/listProbes',
            baseParams: {
                format: 'xml'
            },
            record: 'probe',
            fields: [ 'address', 'port', 'probeName', 'displayName' ],
            listeners: {
                'load': function(store) {
                    store.insert(0, new store.recordType({
                        displayName: 'All',
                        name: '',
                        address: null,
                        port: null
                    }));
                }
            }
        });

        this.comboCacheStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            sortInfo: {
                field: 'name',
                direction: 'ASC'
            },
            baseParams: {
                format: 'xml'
            },
            record: 'name',
            fields: [
                {
                    name: 'name',
                    mapping: '/',
                    type: 'string'
                },
                {
                    name: 'adaptedName',
                    mapping: '/',
                    type: 'string',
                    convert: this.formatCacheName
                }
            ]
        });

        this.summaryStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            baseParams: {
                name: [ 'totalCount', 'memoryCount', 'diskCount' ],
                format: 'xml'
            },
            record: 'sampledStatisticValues',
            fields: [ 'totalCount', 'memoryCount', 'diskCount' ],
            listeners: {
                'load': function(store, records) {
                    this.tabPanel.updateSummaryCounts(records[0].data);
                    this.tabPanel.panelSummary.expand();
                    this.tabPanel.removeAllButton.enable();
                    this.tabPanel.inputSearch.enable();
                    this.tabPanel.tableSearch.enable();
                    this.tabPanel.tableSearchAll.enable();
                    this.tabPanel.panelSearch.expand();
                }
            }
        });

        this.tableSearchStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            url: '/monitor/getCacheKeys',
            sortInfo: {
                field: 'key',
                direction: 'ASC'
            },
            baseParams: {
                count: 100,
                format: 'xml'
            },
            record: 'key',
            fields: [
                {
                    name: 'key',
                    mapping: '/',
                    type: 'string'
                }
            ],
            listeners: {
                'beforeload': function(store) {
                    if (this.tabPanel.tableSearch.hidden) {
                        store.removeAll();
                        this.tabPanel.tableSearch.setVisible(true);
                        this.tabPanel.tableSearchAll.setVisible(false);
                    }
                },
                'load': function(store, records) {
                    this.tabPanel.updateSearchResultCount(records.length);
                    this.tabPanel.panelDetails.collapse();
                }
            }
        });

        this.tableSearchAllStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            url: '/monitor/getAggregatedCacheKeys',
            sortInfo: {
                field: 'key',
                direction: 'ASC'
            },
            baseParams: {
                count: 100,
                format: 'xml'
            },
            record: 'probe',
            fields: [ 'key', 'probeName', 'address', 'port', 'displayName' ],
            listeners: {
                'beforeload': function(store) {
                    if (this.tabPanel.tableSearchAll.hidden) {
                        store.removeAll();
                        this.tabPanel.tableSearch.setVisible(false);
                        this.tabPanel.tableSearchAll.setVisible(true);
                    }
                },
                'load': function(store, records) {
                    this.tabPanel.updateSearchResultCount(records.length);
                    this.tabPanel.panelDetails.collapse();
                }
            }
        });

        this.detailsStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            url: '/monitor/getCacheElementMetaDataAllValues',
            baseParams: {
                format: 'xml'
            },
            record: 'cacheElementMetaDataAllValues',
            fields: [ 'accessed', 'created', 'eternal', 'expired', 'expirationTime', 'hits', 'timeToIdle', 'timeToLive', 'updated', 'valueType' ],
            listeners: {
                'load': function(store, records, options) {
                    records[0].data['key'] = options['scope'];
                    records[0].data['address'] = options['params']['address'];
                    records[0].data['port'] = options['params']['port'];
                    records[0].data['cache'] = options['params']['cache'];
                    this.tabPanel.updateDetails(records[0].data);
                    this.tabPanel.panelDetails.expand();
                }
            }
        });
    },

    createInterface: function() {
        this.comboCacheManager = new TC.components.ComboBox({
            tabPanel: this,
            store: this.comboCacheManagerStore,
            displayField: 'displayName',
            emptyText: 'Select a cache manager...',
            applyTo: 'contents-cache-manager',
            value: 'All',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCacheManager(record.data);
                    if(this.getStore().getCount() == 0){
                        Ext.getCmp('comboCache').disable();
                    }else{
                       Ext.getCmp('comboCache').enable(); 
                    }
                }
            }
        });

        this.comboCache = new TC.components.ComboBox({
            tabPanel: this,
            id: 'comboCache',
            store: this.comboCacheStore,
            displayField: 'adaptedName',
            emptyText: 'Select a cache...',
            applyTo: 'contents-cache',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCache(record.data);

                }
            }
        });

        this.panelSummary = new Ext.Panel({
            title: 'Cache Contents',
            animate: true,
            renderTo: 'contents-summary',
            contentEl: 'contents-summary-table',
            collapsible: true,
            collapsed: true
        });

        this.removeAllButton = new Ext.Button({
            tabPanel: this,
            renderTo: 'contents-remove-all',
            text: 'Remove all',
            disabled: true,
            listeners: {
                'click': function() {
                    this.tabPanel.removeAll();
                }
            }
        });

        this.panelSearch = new Ext.Panel({
            title: 'Search Contents',
            animate: true,
            renderTo: 'contents-search',
            contentEl: 'contents-search-panel',
            collapsible: true,
            collapsed: true
        });

        this.inputSearch = new Ext.form.TextField({
            tabPanel: this,
            applyTo: 'contents-search-query',
            emptyText: 'search for a match in string keys...',
            disabled: true,
            validationDelay: 500,
            listeners: {
                specialkey: function(field, e) {
                    if (e.getKey() == e.ENTER) {
                        this.validate();
                    }
                },

                'valid': function() {
                    this.tabPanel.loadSearchStore();
                }
            }
        });

        this.removeSearchButton = new Ext.Button({
            tabPanel: this,
            renderTo: 'contents-remove-search',
            text: 'Remove results',
            disabled: true,
            listeners: {
                'click': function() {
                    this.tabPanel.removeQuery(this.tabPanel.inputSearch.getValue());
                }
            }
        });

        this.tableSearch = new Ext.grid.GridPanel({
            tabPanel: this,
            store: this.tableSearchStore,
            renderTo: 'contents-search-results-table',
            stateId: 'ehcachedx-contents-search-results-table',
            stateful: true,
            height: 250,
            autoHeight: true,
            autoExpandColumn: 'key',
            disabled: true,
            hidden: true,
            listeners: {
                'cellclick': function(grid, row) {
                    var record = grid.getStore().getAt(row);
                    this.tabPanel.loadDetailsStore(record.data);
                }
            },
            colModel: new Ext.grid.ColumnModel({
                columns: [
                    {
                        id: 'key',
                        header: "Key",
                        dataIndex: 'key',
                        hideable: false,
                        xtype: 'gridcolumn',
                        align: 'left',
                        tooltip: 'The string representation of the keys that match the search query for the selected cache.'
                    }
                ]
            })
        });

        this.tableSearchAll = new Ext.grid.GridPanel({
            tabPanel: this,
            store: this.tableSearchAllStore,
            renderTo: 'contents-search-all-results-table',
            stateId: 'ehcachedx-contents-search-all-table',
            stateful: true,
            height: 300,
            autoHeight: true,
            autoExpandColumn: 'key',
            disabled: true,
            listeners: {
                'cellclick': function(grid, row) {
                    var record = grid.getStore().getAt(row);
                    this.tabPanel.loadDetailsStore(record.data);
                }
            },
            colModel: new Ext.grid.ColumnModel({
                columns: [
                    {
                        id: 'probe',
                        header: "Probe",
                        dataIndex: 'displayName',
                        xtype: 'gridcolumn',
                        align: 'left',
                        width: 300,
                        tooltip: 'The probe this key was found on.'
                    },
                    {
                        id: 'key',
                        header: "Key",
                        dataIndex: 'key',
                        hideable: false,
                        xtype: 'gridcolumn',
                        align: 'left',
                        tooltip: 'The string representation of the keys that match the search query for the selected cache.'
                    }
                ]
            })
        });

        this.panelDetails = new Ext.Panel({
            title: 'Element Details',
            animate: true,
            renderTo: 'contents-details',
            contentEl: 'contents-details-table',
            collapsible: true,
            collapsed: true
        });

        this.removeEntryButton = new Ext.Button({
            tabPanel: this,
            renderTo: 'contents-remove-entry',
            text: 'Remove entry',
            disabled: true,
            listeners: {
                'click': function() {
                    this.tabPanel.removeOne(this.tabPanel.detailsStore.getAt(0).data);
                }
            }
        });
    }
};
