/*
 * All content copyright (c) 2003-2009 Terracotta, Inc., except as may otherwise be noted in a separate copyright notice.  All rights reserved.
 */
Ext.ns('TC.monitor');

TC.monitor.Main = function() {
    this.createViewPort();
    this.createTabs();
    if(this.getURLVar('active') != ''){
        if(Ext.getCmp(this.getURLVar('active')) != null || Ext.getCmp(this.getURLVar('active')) != ""){
            Ext.getCmp('center-region').setActiveTab(this.getURLVar('active'));
       }
    } 
};

TC.monitor.Main.prototype = {
    enabled: true,
    connectionStatus: null,
    viewPort: null,
    progressBar: null,
    tabPanel: null,
    cacheManagersTab: null,
    statisticsTab: null,
    configurationTab: null,
    contentsTab: null,
    cacheEfficiencyTab: null,
    memoryUseTab: null,
    supportedMethodsTab: null,

    doLayout: function() {
        this.viewPort.doLayout();
    },

    createViewPort: function() {
        var monitor = this;

        var handleActivate = function(object) {
            object.main.updateData();
        };

        this.progressBar = new Ext.ProgressBar({
            height: 4,
            renderTo: 'status-progress'

        });

		var V = new Ext.ux.plugin.VisibilityMode({hideMode:'nosize', bubble: true });

        this.viewPort = new Ext.Viewport({
            layout: 'border',
            items: [
                {
                    region: 'north',
                    el: 'north',
                    autoHeight: true,
                    border: false
                },
                {
                    region: 'south',
                    contentEl: 'south',
                    autoHeight: true,
                    collapsible: false
                },

                new Ext.Container({
                    region: 'center',
                    el: 'center',
                    layout: 'anchor',
                    items: [
                        {
                            html: '<div id="connection-severed"><img src="images/error.png" alt="Disconnected" width="16" height="16" ext:qtip="The connection to the server was severed.<br />If the server is started up again, the console GUI will resume its operations." /></div>',
                            id: 'connection-status',
                            border: false,
                            hidden: true
                        },
                        monitor.tabPanel = new Ext.TabPanel({
                            anchor: '100% 100%',
                            id: 'center-region',
                            deferredRender: false,
                            activeTab: 0,
                            plugins: V,
                            defaults: {
                                plugins: V
                            },
                            items: [
                                {
                                    main: this,
                                    id: 'cacheManagers-tab',
                                    contentEl: 'cache-managers',
                                    title: 'Cache Managers',
                                    autoScroll: true,
                                    layoutOnTabChange: true,
                                    listeners: { activate: handleActivate }
                                },
                                {
                                    main: this,
                                    id: 'statistics-tab',
                                    contentEl: 'statistics',
                                    title: 'Statistics',
                                    autoScroll: true,
                                    layoutOnTabChange: true,
                                    listeners: { activate: handleActivate }
                                },
                                {
                                    main: this,
                                    id: 'configuration-tab',
                                    contentEl: 'configuration',
                                    title: 'Configuration',
                                    autoScroll: true,
                                    layoutOnTabChange: true,
                                    listeners: { activate: handleActivate }
                                },
                                {
                                    main: this,
                                    id: 'contents-tab',
                                    contentEl: 'contents',
                                    title: 'Contents',
                                    autoScroll: true,
                                    layoutOnTabChange: true,
                                    listeners: { activate: handleActivate }
                                },
                                {
                                    main: this,
                                    id: 'cacheEfficiency-tab',
                                    contentEl: 'cache-efficiency',
                                    title: 'Charts',
                                    autoScroll: true,
                                    layoutOnTabChange: true,
                                    listeners: { activate: handleActivate }
                                },
//                                {
//                                    main: this,
//                                    id: 'memoryUse-tab',
//                                    contentEl: 'memory-use',
//                                    title: 'Memory Use',
//                                    autoScroll: true,
//                                    layoutOnTabChange: true,
//                                    listeners: { activate: handleActivate }
//                                },
                                {
                                    main: this,
                                    id: 'supportedMethods-tab',
                                    contentEl: 'supported-methods',
                                    title: 'API',
                                    autoScroll: true,
                                    layoutOnTabChange: true,
                                    listeners: { activate: handleActivate }
                                }
                            ],
                            listeners: {
                                'tabchange': function(comp, tab) {
                                     monitor.testCheckStats();
                                 },
                                 scope: this
                            }
                        })
                    ]
                })
            ]
        });

        this.connectionStatus = Ext.getCmp("connection-status");

        Ext.EventManager.onWindowResize(this.updateData, this, { buffer: 300 });
    },

    testCheckStats: function() {
        if (this.tabPanel.getActiveTab() === Ext.getCmp('statistics-tab') ||
            this.tabPanel.getActiveTab() === Ext.getCmp('cacheEfficiency-tab')) {
            this.checkStats();
        }
    },
    
    createTabs: function() {
        this.cacheManagersTab = new TC.monitor.tabs.CacheManagers(this);
        this.statisticsTab = new TC.monitor.tabs.Statistics(this);
        this.configurationTab = new TC.monitor.tabs.Configuration(this);
        this.contentsTab = new TC.monitor.tabs.Contents(this);
        this.cacheEfficiencyTab = new TC.monitor.tabs.CacheEfficiency(this);
        //this.memoryUseTab = new TC.monitor.tabs.MemoryUse(this);
        this.supportedMethodsTab = new TC.monitor.tabs.SupportedMethods(this);
    },

    updateCacheManagersTab: function () {
        if (this.cacheManagersTab !== null && this.tabPanel.getActiveTab() === Ext.getCmp('cacheManagers-tab')) {
            this.cacheManagersTab.updateData();
        }
    },

    updateStatisticsTab: function () {
        if (this.statisticsTab !== null && this.tabPanel.getActiveTab() === Ext.getCmp('statistics-tab')) {
            this.statisticsTab.updateData();
        }
    },

    updateConfigurationTab: function () {
        if (this.configurationTab !== null && this.tabPanel.getActiveTab() === Ext.getCmp('configuration-tab')) {
            this.configurationTab.updateData();
        }
    },

    updateContentsTab: function () {
        if (this.contentsTab !== null && this.tabPanel.getActiveTab() === Ext.getCmp('contents-tab')) {
            this.contentsTab.updateData();
        }
    },

    updateCacheEfficiencyTab: function () {
        if (this.cacheEfficiencyTab !== null && this.tabPanel.getActiveTab() === Ext.getCmp('cacheEfficiency-tab')) {
            this.cacheEfficiencyTab.updateData();
        }
    },

    updateMemoryUseTab: function () {
        if (this.memoryUseTab !== null && this.tabPanel.getActiveTab() === Ext.getCmp('memoryUse-tab')) {
            this.memoryUseTab.updateData();
        }
    },

    updateSupportedMethodsTab: function () {
        if (this.supportedMethodsTab !== null && this.tabPanel.getActiveTab() === Ext.getCmp('supportedMethods-tab')) {
            this.supportedMethodsTab.updateData();
        }
    },

    checkRestApi: function(updateMillis) {
        var monitor = this;
        Ext.Ajax.request({
            url: '/monitor/isRestApiActive',
            failure: function(response) {
                if (402 == response.status) {
                    monitor.enabled = false;
                    Ext.Msg.show({
                        title: 'Product key expired',
                        msg: 'Your Ehcache Monitor product key has expired.<br /><br />' +
                             'The Ehcache monitoring probe capability will be suspended until restarted, Ehcache will ' +
                             'not be affected.<br /><br />' +
                             'For a license for use in production please contact <a href="mailto:sales@terracottatech.com">sales@terracottatech.com</a>.',
                        width: 500,
                        closable: false,
                        icon: Ext.MessageBox.INFO
                    })
                } else {
                    monitor.connectionStatus.setVisible(true);
                }
            },
            success: function() {
                monitor.connectionStatus.setVisible(false);
                monitor.progressBar.reset();
                monitor.progressBar.wait({
                    interval: 1000,
                    animate: true,
                    duration: updateMillis,
                    increment: updateMillis / 1000,
                    text: '',
                    scope: this
                });

                /*monitor.progressBar.on("update", function(e,n,s){
                    if(n == 1){
                        monitor.updateCacheManagersTab();
                        monitor.updateStatisticsTab();
                        monitor.updateConfigurationTab();
                        monitor.updateContentsTab();
                        monitor.updateCacheEfficiencyTab();
                        monitor.updateMemoryUseTab();
                        monitor.updateSupportedMethodsTab();
                    }
                });  */

                monitor.updateCacheManagersTab();
                monitor.updateStatisticsTab();
                monitor.updateConfigurationTab();
                monitor.updateContentsTab();
                monitor.updateCacheEfficiencyTab();
                monitor.updateMemoryUseTab();
                //monitor.updateSupportedMethodsTab();
            }
        });
    },

    updateData: function(updateMillis) {
        if (this.enabled) {
            this.checkRestApi(updateMillis);
        }
    },

    showCacheManagersTab: function() {
        this.tabPanel.setActiveTab(0);
    },

    showStatisticsTab: function() {
        this.tabPanel.setActiveTab(1);
    },

    showConfigurationTab: function() {
        this.tabPanel.setActiveTab(2);
    },

    showContentsTab: function() {
        this.tabPanel.setActiveTab(3);
    },

    showCacheEfficiencyTab: function() {
        this.tabPanel.setActiveTab(4);
    },

    showMemoryUseTab: function() {
        this.tabPanel.setActiveTab(5);
    },

    showSupportedMethodsTab: function() {
        this.tabPanel.setActiveTab(6);
    },

    changeActiveCacheManager: function(data) {
        if (this.statisticsTab !== null) {
            this.statisticsTab.changeActiveCacheManager(data);
        }
        if (this.configurationTab !== null) {
            this.configurationTab.changeActiveCacheManager(data);
        }
        if (this.contentsTab !== null) {
            this.contentsTab.changeActiveCacheManager(data);
        }
        if (this.cacheEfficiencyTab !== null) {
            this.cacheEfficiencyTab.changeActiveCacheManager(data);
        }
        if (this.memoryUseTab !== null) {
            this.memoryUseTab.changeActiveCacheManager(data);
        }
    },

    changeActiveCache: function(data) {
        if (this.contentsTab !== null) {
            this.contentsTab.changeActiveCache(data);
        }
        if (this.cacheEfficiencyTab !== null) {
            this.cacheEfficiencyTab.changeActiveCache(data);
        }
        if (this.memoryUseTab !== null) {
            this.memoryUseTab.changeActiveCache(data);
        }
    },

    getURLVar: function(urlVarName){
               //divide the URL in half at the '?' to simulate PHP variables.
                var urlHalves = String(document.location).split('?');
                var urlVarValue = '';
                if(urlHalves[1]){
                    var urlVars = urlHalves[1].split('&');

                    for(var i=0; i<=(urlVars.length); i++){

                        if(urlVars[i]){
                            var urlVarPair = urlVars[i].split('=');

                            if (urlVarPair[0] && urlVarPair[0] == urlVarName) {
                                urlVarValue = urlVarPair[1];
                            }
                        }
                    }
                }
                return urlVarValue;
    },
    
    checkStats: function() {
        Ext.Ajax.request({
            url: '/monitor/getAggregatedStatisticsState',
            failure: function(response) {
                Ext.Msg.show({
                    title: 'Error while determining cache statistics state',
                    msg: 'An unexpected error occurred while determining the state of the statistics gathering for all caches',
                    autoWidth: true,
                    closable: true,
                    buttons: Ext.Msg.OK,
                    icon: Ext.MessageBox.ERROR
                });
            },
            success: function(response) {
                if (response.responseText == 'false') {
                    Ext.Msg.show({
                        title: 'Ehcache Monitor',
                        msg: 'Statistics are not enabled for all caches.  Enable all statistics?',
                        buttons: Ext.MessageBox.YESNO,
                        fn: function(btn, text) {
                            if (btn == 'yes') {
                                Ext.Ajax.request({
                                    url: '/monitor/enableAllCacheStatistics',
                                    failure: function(response) {
                                        Ext.Msg.show({
                                            title: 'Error while enabling all cache statistics',
                                            msg: 'An unexpected error occurred while enabling all cache statistics',
                                            autoWidth: true,
                                            closable: true,
                                            buttons: Ext.Msg.OK,
                                            icon: Ext.MessageBox.ERROR
                                        });
                                    }
                                });
                            }
                        },
                        icon: Ext.MessageBox.QUESTION
                    });
                }
            }
        });
    }
};
