/*
 * All content copyright (c) 2003-2009 Terracotta, Inc., except as may otherwise be noted in a separate copyright notice.  All rights reserved.
 */
Ext.ns('TC.monitor.tabs');

TC.monitor.tabs.MemoryUse = function(monitor) {
    this.monitor = monitor;
    this.createStores();
    alert('The Memory Use tab hasn\'t been debugged for IE8 yet');
    this.createInterface();

};

// TODO Update this based on recent changes in CacheEfficiency.
TC.monitor.tabs.MemoryUse.prototype = {
    monitor: null,

    cacheManagerAddress: null,
    cacheManagerPort: null,
    cacheName: null,

    comboCacheManager: null,
    comboCache: null,
    chart: null,
    chartTitleSuffix: '',
    timeAxisFormat: 'H:i:s',
    forwardButton: null,
    backButton: null,
    settingsButton: null,
    settingsWindow: null,

    comboCacheManagerStore: null,
    comboCacheStore: null,
    chartStore: null,
    live: true,
    fromDate: null,
    fromTime: null,
    timeInterval: 60,

    loadCacheStore: function() {
        if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null) {
            this.comboCacheStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getCacheNames'});
            this.comboCacheStore.load({
                params: {
                    'address': this.cacheManagerAddress,
                    'port': this.cacheManagerPort
                }
            });
        }
    },

    loadChartStore: function(offset) {
        if (this.chartStore !== null && this.chart !== null) {
            if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null && this.cacheName !== null) {
                var params = {
                    'address': this.cacheManagerAddress,
                    'port': this.cacheManagerPort,
                    'cache': this.cacheName
                };
                if (!this.live && this.fromDate != null && this.fromDate != "" && this.fromTime != null && this.fromTime != "") {
                    params['fromDateTime'] = this.fromDate.format('Y/m/d') + " " + this.fromTime;
                }
                if (this.timeInterval != null) {
                    params['timeInterval'] = this.timeInterval;
                }
                if (offset != null) {
                    params['offset'] = offset;
                }
                this.chartStore.load({
                    params: params,
                    scope: this,
                    callback: function() {
                        if (this.chartStore != null) {
                            var firstDateTime = this.chartStore.getAt(0).get('time');
                            this.fromTime = firstDateTime.format("H:i:s");
                            this.fromDate = firstDateTime.clearTime();
                        }
                    }
                });
            }
        }
    },

    clearSelectedCache: function() {
        this.cacheName = null;
        this.comboCache.clearValue();
    },

    updateChartData: function(offset) {
        if (Ext.get('memoryUse-tab').isVisible()) {
            this.loadChartStore(offset);
            this.adjustChartTimeAxis();

            if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null && this.cacheName !== null) {
                if (offset == 'back') {
                    this.live = false;
                }
                if (this.live) {
                    this.forwardButton.disable();
                    this.backButton.enable();
                } else {
                    this.forwardButton.enable();
                    this.backButton.enable();
                }
            }
        }
    },

    adjustChartTimeAxis: function() {
        if (this.timeInterval == 60) {
            this.timeAxisFormat = "H:i:s";
        } else if (this.timeInterval == 300) {
            this.timeAxisFormat = "H:i:s";
        } else if (this.timeInterval == 1800) {
            this.timeAxisFormat = "H:i";
        } else if (this.timeInterval == 3600) {
            this.timeAxisFormat = "H:i";
        } else if (this.timeInterval == 86400) {
            this.timeAxisFormat = "H";
        }
    },

    updateData: function() {
        if (Ext.get('memoryUse-tab').isVisible()) {
            if (!this.comboCacheManager.isExpanded()) {
                this.comboCacheManagerStore.load();
            }
            if (!this.comboCache.isExpanded()) {
                this.loadCacheStore();
            }
            if (this.live) {
                this.updateChartData();
            }
        }
    },

    changeActiveCacheManager: function(data) {
        if (this.cacheManagerAddress != data['address'] || this.cacheManagerPort != data['port']) {
            this.cacheManagerAddress = data['address'];
            this.cacheManagerPort = data['port'];
            this.clearSelectedCache();
            this.loadCacheStore();
            this.comboCacheManager.setValue(data['displayName'])
        }
    },

    changeActiveCache: function(data) {
        if (this.cacheName != data['name']) {
            this.cacheName = data['name'];
            this.updateChartData();
            this.comboCache.setValue(this.formatCacheName(data['name']))
        }
    },

    formatCacheName: function(fullName) {
        // anything shorter than 37 chars will not be shortened
        if (fullName.length < 37) {
            return fullName;
        } else {
            var result = '';

            var comps = fullName.split('.');
            var truncate = true;
            for (var i = 0; i < comps.length; i++) {
                var comp = comps[i];
                if (truncate &&
                    comp.charCodeAt(0) < 97) { // check if it's uppercase by looking at the 'A' ascii code
                    truncate = false;
                }

                if (result.length > 0) {
                    result += '.';
                }

                if (truncate) {
                    result += comp.charAt(0);
                } else {
                    result += comp;
                }
            }

            return result;
        }
    },

    createStores: function() {
        this.comboCacheManagerStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: true,
            autoDestroy: true,
            sortInfo: {
                field: 'displayName',
                direction: 'ASC'
            },
            url: '/monitor/listProbes',
            baseParams: {
                format: 'xml'
            },
            record: 'probe',
            fields: [ 'address', 'port', 'probeName', 'displayName' ]
        });

        this.comboCacheStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            sortInfo: {
                field: 'name',
                direction: 'ASC'
            },
            baseParams: {
                format: 'xml'
            },
            record: 'name',
            fields: [
                {
                    name: 'name',
                    mapping: '/',
                    type: 'string'
                },
                {
                    name: 'adaptedName',
                    mapping: '/',
                    type: 'string',
                    convert: this.formatCacheName
                }
            ]
        });
    },

    createChartStore: function() {
        this.chartStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            url: '/monitor/getSampledStatisticHistory',
            baseParams: {
                name: 'hitRatio',
                format: 'xml'
            },
            record: 'sampledStatisticHistory > sample',
            fields: [
                {name: 'hitRatio', mapping: 'value', type: 'float'},
                {name: 'time', mapping: 'timestamp', type: 'date', dateFormat: 'Y/m/d H:i:s'}
            ]
        });
    },

    createChart: function() {
        var tabPanel = this;

        this.chart = new Ext.Panel({
            tabPanel: this,
            iconCls: 'chart',
            title: 'Cache Hits/Total Hits (%) ' + this.chartTitleSuffix,
            frame: true,
            applyTo: 'memory-use-chart',
            width: 800,
            height: 300,
            layout: 'fit',
            items: {
                xtype: 'linechart',
                store: this.chartStore,
                xField: 'time',
                yField: 'hitRatio',
                xAxis: new Ext.chart.TimeAxis({
                    title: 'Time',
                    labelRenderer: function(date) {
                        return date.format(tabPanel.timeAxisFormat);
                    }
                }),
                yAxis: new Ext.chart.NumericAxis({
                    title: 'Cache Hit Ratio (%)',
                    minimum: 0,
                    maximum: 100,
                    labelRenderer: Ext.util.Format.numberRenderer('0,0')
                })
            }
        });
    },

    recreateChartInterface: function() {
        if (this.chartStore !== null) {
            this.chartStore.destroy();
            this.chartStore = null;
        }

        if (this.chart !== null) {
            this.chart.hide();
            this.chart.destroy();
            this.chart = null;
        }
        alert('2.1.1.1');
        this.createChartStore();
        alert('2.1.1.2');
        this.createChart();
        alert('2.1.1.3');
        this.chart.show();
        alert('2.1.1.4');
    },

    createSettingsInterface: function() {
        var liveCheckbox = new Ext.form.Checkbox({
            fieldLabel: 'Live',
            boxLabel: '',
            name: 'live',
            checked: true
        });

        var fromDateField = new Ext.form.DateField({
            fieldLabel: 'From',
            name: 'fromDate',
            format: 'Y/m/d'
        });

        var fromTimeField = new Ext.form.TimeField({
            name: 'fromTime',
            increment: 1,
            format: 'H:i'
        });

        var formPanel = new Ext.form.FormPanel({
            tabPanel: this,
            labelWidth: 60,
            frame: false,
            bodyStyle: 'padding: 5px 5px 0',
            width: 500,
            autoHeight: true,
            items: [
                liveCheckbox,
                fromDateField,
                fromTimeField,
                new Ext.form.ComboBox({
                    name: 'timeInterval',
                    fieldLabel: 'Time Interval',
                    width: 360,
                    mode: 'local',
                    triggerAction: 'all',
                    selectOnFocus: false,
                    forceSelection: true,
                    typeAhead: true,
                    triggerClass: 'monitor-combo-trigger-360',
                    store: [[60, '1 minute'], [300, '5 minutes'], [1800, '30 minutes'], [3600, '1 hour'], [86400, '1 day']],
                    value: 60
                })
            ],
            buttons: [
                {
                    text: 'OK',
                    scope: this,
                    handler: function() {
                        this.settingsWindow.hide();

                        var values = formPanel.getForm().getFieldValues();
                        this.live = values['live'];
                        this.fromDate = values['fromDate'];
                        var fromTime = values['fromTime'];
                        if (fromTime == null || fromTime == "") {
                            fromTime = "00:00";
                        }
                        this.fromTime = fromTime + ":00";
                        this.timeInterval = values['timeInterval'];
                        this.updateChartData();
                    }
                },
                {
                    text: 'Cancel',
                    scope: this,
                    handler: function() {
                        this.settingsWindow.hide();
                    }
                }
            ]
        });

        this.settingsWindow = new Ext.Window({
            tabPanel: this,
            animCollapse: true,
            animateTarget: 'memory-use-settings',
            title: 'Chart Settings',
            resizable: false,
            layout: 'fit',
            autoWidth: true,
            autoHeight: true,
            closable: false,
            modal: true,
            shim: true,
            buttonAlign: 'center',
            closeAction: 'hide',
            items: [
                formPanel
            ]
        });

        this.settingsButton = new Ext.Button({
            scope: this,
            renderTo: 'memory-use-settings',
            text: 'Chart Settings',
            handler: function() {
                formPanel.getForm().setValues({
                    live: this.live,
                    fromDate: this.fromDate,
                    fromTime: this.fromTime,
                    timeInterval: this.timeInterval
                });
                this.settingsWindow.show('memory-use-settings');
            }
        });
    },

    createInterface: function() {

           alert('2.1.0');
        this.recreateChartInterface();
        alert('2.1.1');
        this.createSettingsInterface();
       alert('2.1.2');
        this.comboCacheManager = new TC.components.ComboBox({
            tabPanel: this,
            store: this.comboCacheManagerStore,
            displayField: 'displayName',
            emptyText: 'Select a cache manager...',
            applyTo: 'memory-use-cache-manager',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCacheManager(record.data);
                }
            }
        });

        this.comboCache = new TC.components.ComboBox({
            tabPanel: this,
            store: this.comboCacheStore,
            displayField: 'adaptedName',
            emptyText: 'Select a cache...',
            applyTo: 'memory-use-cache',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCache(record.data);
                }
            }
        });

        this.forwardButton = new Ext.Button({
            tabPanel: this,
            renderTo: 'memory-use-forward',
            text: 'Forward',
            handler: function() {
                this.tabPanel.updateChartData('forward');
            }
        });
        this.forwardButton.disable();

        this.backButton = new Ext.Button({
            tabPanel: this,
            renderTo: 'memory-use-back',
            text: 'Back',
            handler: function() {
                this.tabPanel.updateChartData('back');
            }
        });
        this.backButton.disable();
    }
};
