/*
 * All content copyright (c) 2003-2009 Terracotta, Inc., except as may otherwise be noted in a separate copyright notice.  All rights reserved.
 */
Ext.ns('TC.monitor.tabs');

TC.monitor.tabs.Statistics = function(monitor) {
    this.monitor = monitor;
    this.selectedStatistics = Ext.state.Manager.get("ehcachedx-selected-statistics");
    if (Ext.isEmpty(this.selectedStatistics)) {
        this.selectedStatistics = [
            'totalCount',
            'hitRatio',
            'hit',
            'miss',
            'put'
        ]
    }

    this.createStores();


    this.createInterface();
};

TC.monitor.tabs.Statistics.prototype = {
    monitor: null,

    cacheManagerAddress: null,
    cacheManagerPort: null,

    comboCacheManager: null,
    settingsButton: null,
    settingsWindow: null,
    table: null,

    comboCacheManagerStore: null,
    tableStore: null,
    settingsStore: null,

    selectedStatistics: null,

    statisticsSettingsGroups: [
        {
            id: 'active-statistics-size',
            title: 'Size',
            stats: [
                'totalCount',
                'memoryCount',
                'offHeapCount',
                'diskCount'
            ]
        },
        {
            id: 'active-statistics-offheap',
            title: 'OffHeap',
            stats: [
                'offHeapUsed',
                'offHeapSize',
                'offHeapUsage'
            ]
        },
        {
            id: 'active-statistics-efficiency',
            title: 'Cache efficiency',
            stats: [
                'hitRatio',
                'hit',
                'hitOffHeap',
                'hitDisk',
                'hitMemory',
                'miss',
                'missOffHeap',
                'missDisk',
                'missMemory'
            ]
        },
        {
            id: 'active-statistics-modifications',
            title: 'Cache modifications',
            stats: [
                'put',
                'removed',
                'updated'
            ]
        },
        {
            id: 'active-statistics-eviction',
            title: 'Eviction rates',
            stats: [
                'evicted',
                'expired'
            ]
        }
    ],

    statisticsLabels: {
        diskCount: 'Disk Elements',
        evicted: 'Evicted/s',
        expired: 'Expired/s',
        hit: 'Hits/s',
        hitOffHeap: 'OffHeap Hits/s',
        hitDisk: 'Disk Hits/s',
        hitMemory: 'Mem Hits/s',
        hitRatio: 'Hit Ratio',
        memoryCount: 'Mem Elements',
        miss: 'Misses/s',
        missOffHeap: 'OffHeap Misses/s',
        missDisk: 'Disk Misses/s',
        missMemory: 'Mem Misses/s',
        put: 'Puts/s',
        removed: 'Removes/s',
        totalCount: 'Total Elements',
        updated: 'Updates/s',
        offHeapCount: 'OffHeap Elements',
        offHeapUsed: 'OffHeap Used',
        offHeapSize: 'OffHeap Max',
        offHeapUsage: 'OffHeap Usage'
    },

    statisticsTooltips: {
        diskCount: 'The number of elements in the Disk Store.',
        evicted: 'The number of elements that were evicted.',
        expired: 'The number of elements that were expired.',
        hit: 'The number of times a requested item was found in the cache.',
        hitOffHeap: 'The number of times a requested item was found in the OffHeap Store.',
        hitDisk: 'The number of times a requested item was found in the Disk Store.',
        hitMemory: 'The number of times a requested item was found in the Memory Store.',
        hitRatio: 'The percentage of requested items that were found in the cache.',
        memoryCount: 'The number of elements in the Memory Store.',
        miss: 'The number of times a requested item was not found in the cache.',
        missOffHeap: 'The number of times a requested item was not found in the OffHeap Store.',
        missDisk: 'The number of times a requested item was not found in the Disk Store.',
        missMemory: 'The number of times a requested item was not found in the Memory Store.',
        put: 'The number of elements that were put into the cache.',
        removed: 'The number of elements that removed from the cache.',
        totalCount: 'The number of elements in the cache.',
        updated: 'The number of elements that were put in the cache when they already existed.',
        offHeapCount: 'The number of elements that are being kept OffHeap.',
        offHeapUsed: 'The amount of OffHeap storage in use (bytes)',
        offHeapSize: 'The maximum available OffHeap storage (bytes)',
        offHeapUsage: 'The relative amount of OffHeap storage in use'
    },

    statisticsFormat: {
        evicted: '0,000.0',
        expired: '0,000.0',
        hit: '0,000.0',
        hitOffHeap: '0,000.0',
        hitDisk: '0,000.0',
        hitMemory: '0,000.0',
        hitRatio: '0,000.0%',
        miss: '0,000.0',
        missOffHeap: '0,000.0',
        missDisk: '0,000.0',
        missMemory: '0,000.0',
        put: '0,000.0',
        removed: '0,000.0',
        updated: '0,000.0',
        offHeapUsage: '0,000.0%'
    },

    statisticsType: {
        diskCount: 'int',
        evicted: 'float',
        expired: 'float',
        hit: 'float',
        hitOffHeap: 'float',
        hitDisk: 'float',
        hitMemory: 'float',
        hitRatio: 'float',
        memoryCount: 'int',
        miss: 'float',
        missOffHeap: 'float',
        missDisk: 'float',
        missMemory: 'float',
        put: 'float',
        removed: 'float',
        totalCount: 'int',
        updated: 'float',
        offHeapCount: 'int',
        offHeapUsed: 'long',
        offHeapSize: 'long',
        offHeapUsage: 'float'
    },

    loadTableStore: function() {
        if (this.tableStore !== null && this.table !== null) {
            this.tableStore.remove();
            if (this.cacheManagerAddress !== null && this.cacheManagerPort !== null) {
                this.tableStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getAllCachesSampledStatisticValues'});
                this.tableStore.load({
                    params: {
                        'address': this.cacheManagerAddress,
                        'port': this.cacheManagerPort
                    }
                });
            } else if (this.cacheManagerAddress === null && this.cacheManagerPort === null) {
                this.tableStore.proxy = new Ext.data.HttpProxy({url: '/monitor/getAggregatedSampledStatisticValuesPerCache'});
                this.tableStore.load();
            }
        }
    },

    updateData: function() {
        if (Ext.get('statistics-tab').isVisible()) {
            if (!this.comboCacheManager.isExpanded()) {
                this.comboCacheManagerStore.load();
            }
            this.loadTableStore();
        }
    },

    changeActiveCacheManager: function(data) {
        if (this.cacheManagerAddress != data['address'] || this.cacheManagerPort != data['port']) {
            this.cacheManagerAddress = data['address'];
            this.cacheManagerPort = data['port'];
            this.loadTableStore();
            this.comboCacheManager.setValue(data['displayName'])
        }
    },

    createStores: function() {
        this.comboCacheManagerStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: true,
            autoDestroy: true,
            sortInfo: {
                field: 'displayName',
                direction: 'ASC'
            },
            url: '/monitor/listProbes',
            baseParams: {
                format: 'xml'
            },
            record: 'probe',
            fields: [ 'address', 'port', 'probeName', 'displayName' ],
            listeners: {
                'load': function(store) {
                    store.insert(0, new store.recordType({
                        displayName: 'All',
                        name: '',
                        address: null,
                        port: null
                    }));
                }
            }
        });
    },

    createStatisticsStore: function() {
        var tableStoreFields = new Array();
        tableStoreFields.push({
            name: 'name',
            type: 'string'
        });
        tableStoreFields.push({
            name: 'cacheManagerCount',
            type: 'int'
        });
        for (var i = 0; i < this.selectedStatistics.length; i++) {
            var name = this.selectedStatistics[i];
            tableStoreFields.push({
                name: name,
                mapping: 'sampledStatisticValues > ' + name,
                type: this.statisticsType[name]
            });
        }

        this.tableStore = new Ext.data.XmlStore({
            tabPanel: this,
            autoLoad: false,
            autoDestroy: true,
            sortInfo: {
                field: 'name',
                direction: 'ASC'
            },
            baseParams: {
                name: this.selectedStatistics,
                format: 'xml'
            },
            record: 'cache',
            fields: tableStoreFields,
            listeners: {
                'load': function() {
                    this.tabPanel.updateStatisticsTableLayout();
                }
            }
        });
    },

    updateStatisticsTableLayout: function() {
        if (this.table !== null) {
            var columnModel = this.table.getColumnModel();
            var columnIndex = columnModel.getIndexById('cacheManagerCount');
            columnModel.setHidden(columnIndex, this.shouldCacheManagerCountBeHidden());
        }
    },

    shouldCacheManagerCountBeHidden: function() {
        return this.cacheManagerAddress !== null || this.cacheManagerPort !== null;
    },

    createStatisticsTable: function() {
        // Dynamically create the columns that the table should display.
        var tableColumns = new Array();
        tableColumns.push({
            id: 'name',
            header: "Cache",
            dataIndex: 'name',
            hideable: false,
            xtype: 'gridcolumn',
            align: 'left',
            tooltip: 'The name of the cache.'
        });

        tableColumns.push({
            id: 'cacheManagerCount',
            header: "Cache Instances",
            dataIndex: 'cacheManagerCount',
            xtype: 'numbercolumn',
            align: 'right',
            width: 120,
            tooltip: 'The number of cache managers that have this cache.',
            hidden: this.shouldCacheManagerCountBeHidden()
        });
        for (var i = 0; i < this.selectedStatistics.length; i++) {
            var name = this.selectedStatistics[i];
            tableColumns.push({
                dataIndex: name,
                header: this.statisticsLabels[name],
                format: this.statisticsFormat[name],
                tooltip: this.statisticsTooltips[name]
            });
        }
        this.table = new Ext.grid.GridPanel({
            tabPanel: this,
            store: this.tableStore,
            renderTo: 'statistics-table',
            stateId: 'ehcachedx-statistics-table',
            stateful: true,
            autoExpandColumn: 'name',
            //height: 500,
            autoHeight: true,
            listeners: {
                'cellclick': function(grid, row) {
                    var record = grid.getStore().getAt(row);
                    this.tabPanel.monitor.changeActiveCache(record.data);
                    this.tabPanel.monitor.showContentsTab();
                }
            },
            colModel: new Ext.grid.ColumnModel({
                defaults: {
                    width: 100,
                    sortable: true,
                    xtype: 'numbercolumn',
                    format: '0,000',
                    align: 'right'
                },
                columns: tableColumns
            })
        });
    },

    createStatisticsInterface: function() {
        if (this.tableStore !== null) {
            this.tableStore.destroy();
            this.tableStore = null;
        }

        if (this.table !== null) {
            this.table.hide();
            this.table.destroy();
            this.table = null;
        }

        this.createStatisticsStore();

        this.createStatisticsTable();

        this.table.show();
    },

    createSettingsInterface: function() {
        var groupItems = new Array();

        for (var i = 0; i < this.statisticsSettingsGroups.length; i++) {
            var settingsItems = new Array();

            var stats = this.statisticsSettingsGroups[i]['stats'];
            for (var j = 0; j < stats.length; j++) {
                var name = stats[j];
                settingsItems.push({
                    name: name,
                    fieldLabel: this.statisticsLabels[name],
                    checked: this.selectedStatistics.indexOf(name) != -1
                });
            }
            groupItems.push({
                id: this.statisticsSettingsGroups[i]['id'],
                xtype: 'fieldset',
                title: this.statisticsSettingsGroups[i]['title'],
                autoHeight: true,
                defaultType: 'checkbox',
                items: settingsItems
            });
        }

        var statisticsTab = this;
        this.settingsWindow = new Ext.Window({
            animCollapse: true,
            animateTarget: 'statistics-settings',
            title: 'Statistics Settings',
            resizable: false,
            closable: false,
            modal: true,
            shim: true,
            buttonAlign: 'center',
            closeAction: 'hide',
            layout: 'fit',
            autoScroll: true,
            height: 500,
            width: 300,
            items: [
                new Ext.form.FormPanel({
                    labelWidth: 120,
                    frame: false,
                    bodyStyle: 'padding: 5px 5px 0',
                    autoHeight: true,
                    items: groupItems
                })
            ],
            buttons: [
                {
                    text: 'OK',
                    listeners: {
                        'click': function() {
                            var selectedStatisticsNew = new Array();

                            for (var i = 0; i < statisticsTab.statisticsSettingsGroups.length; i++) {
                                Ext.getCmp(statisticsTab.statisticsSettingsGroups[i]['id']).items.each(function(item) {
                                    if (item.getValue()) {
                                        selectedStatisticsNew.push(item.name);
                                    }
                                });
                            }

                            statisticsTab.settingsWindow.hide();

                            statisticsTab.selectedStatistics = selectedStatisticsNew;
                            Ext.state.Manager.set("ehcachedx-selected-statistics", selectedStatisticsNew);

                            statisticsTab.createStatisticsInterface();
                            statisticsTab.updateData();
                        }
                    }
                },
                {
                    text: 'Cancel',
                    listeners: {
                        'click': function() {
                            for (var i = 0; i < statisticsTab.statisticsSettingsGroups.length; i++) {
                                Ext.getCmp(statisticsTab.statisticsSettingsGroups[i]['id']).items.each(function(item) {
                                    if (statisticsTab.selectedStatistics.indexOf(item.name) == -1) {
                                        item.setValue(false);
                                    } else {
                                        item.setValue(true);
                                    }
                                });
                            }

                            statisticsTab.settingsWindow.hide();
                        }
                    }
                }
            ]
        });

        this.settingsButton = new Ext.Button({
            renderTo: 'statistics-settings',
            text: 'Settings',
            listeners: {
                'click': function() {
                    statisticsTab.settingsWindow.show();
                }
            }
        });
    },

    createInterface: function() {

        this.createStatisticsInterface();

        this.createSettingsInterface();

        this.comboCacheManager = new TC.components.ComboBox({
            tabPanel: this,
            store: this.comboCacheManagerStore,
            displayField: 'displayName',
            emptyText: 'Select a cache manager...',
            applyTo: 'statistics-cache-manager',
            value: 'All',
            listeners: {
                'select': function(combo, record) {
                    this.tabPanel.monitor.changeActiveCacheManager(record.data);

                }
            }
        });
    }
};
